using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using SMPP;
using SMPP.Packet;
using SMPP.Packet.Request;
using SMPP.Packet.Response;

namespace SMPPClient
{
    public partial class SmppClient : Form
    {
        SMPPCommunicator com;
        Configuration config;
        string configPath;

        public SmppClient()
        {
            InitializeComponent();
            com = new SMPPCommunicator();
            config = new Configuration();
            string applicationPath = Application.ExecutablePath;
            configPath = applicationPath.Replace("exe", "config");
            config = Configuration.Deserialize(@configPath);
            com.OnBindResp += new SMPPCommunicator.BindRespEventHandler(com_OnBindResp);
            com.OnUnboundResp += new SMPPCommunicator.UnbindRespEventHandler(com_OnUnbindResp);
            com.OnSubmitSmResp +=new SMPPCommunicator.SubmitSmRespEventHandler(com_OnSubmitSmResp);
            
            textBoxSourceTon.Text = config.SourceTon.ToString();
            textBoxSourceNpi.Text = config.SourceNpi.ToString();
            textBoxSourceAddress.Text = config.SourceAddress;
            textBoxDestinationTon.Text = config.DestinationTon.ToString();
            textBoxDestinationNpi.Text = config.DestinationNpi.ToString();
            textBoxDestinationAddress.Text = config.DestinationAddress;
            textBoxHost.Text = config.Host;
            textBoxPort.Text = config.Port.ToString();
            textBoxSystemId.Text = config.SystemId;
            textBoxPassword.Text = config.Password;

            if (config.ReplaceIfPresent == "true") checkBoxReplaceIfPresent.Checked = true;
            if (config.RegisterDelivery == "true") checkBoxRegisterDelivery.Checked = true;
        }

        private void com_OnBindResp(object sender, SMPP.EventObject.BindRespEventArgs args)
        {
            uint status = args.ResponsePdu.CommandStatus;
            if (status == 0) toolStripStatusLabel1.Text = "Connected";
            
        }
        private void com_OnUnbindResp(object sender, SMPP.EventObject.UnbindRespEventArgs args)
        {
            uint status = args.ResponsePdu.CommandStatus;
            if (status == 0) toolStripStatusLabel1.Text = "Disconnected";
        }
        private void com_OnSubmitSmResp(object sender, SMPP.EventObject.SubmitSmRespEventArgs args)
        {
            string status = args.ResponsePdu.SequenceNumber.ToString();
            
        }
        
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void editToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }

        private void SmppClient_Load(object sender, EventArgs e)
        {

        }

        private void buttonBind_Click(object sender, EventArgs e)
        {
            com.Host = textBoxHost.Text;
            com.Port = Int16.Parse(textBoxPort.Text);
            com.SystemId = textBoxSystemId.Text;
            com.Password = textBoxPassword.Text;
            switch(config.BindType)
            {
                case "Transceiver":
                    com.BindType = SmppBind.BindingType.BindAsTransceiver;
                    break;
                case "Trasmitter":
                    com.BindType = SmppBind.BindingType.BindAsTrasmitter;
                    break;
                case "Receiver":
                    com.BindType = SmppBind.BindingType.BindAsReceiver;
                    break;
                default: 
                    com.BindType = SmppBind.BindingType.BindAsTransceiver;
                    break;
            }
            com.EnquireLinkInterval = Int16.Parse(config.EnquireLinkInterval.ToString());

            com.Bind();
        }

        private void toolStripStatusLabel_Click(object sender, EventArgs e)
        {

        }

        private void buttonUnbind_Click(object sender, EventArgs e)
        {
            com.Unbind();
        }

        private void textBoxMessage_TextChanged(object sender, EventArgs e)
        {
            labelCounter.Text = textBoxMessage.Text.Length.ToString();
        }

        #region Validating
        private void textBoxSourceTon_Validating(object sender, CancelEventArgs e)
        {
            try
            {
                UInt16 value = UInt16.Parse(textBoxSourceTon.Text);
                if (value > 6)
                {
                    throw new FormatException();
                }
                errorProvider.SetError(textBoxSourceTon, "");
            }
            catch (FormatException)
            {
                errorProvider.SetError(textBoxSourceTon, "Invalid Ton\n0-Unknown\n1-International\n2-National\n3-Network Specific\n4-Subscriber Number\n5-Alphanumeric\n6-Abbriviated");
            }
        }

        private void textBoxSourceNpi_Validating(object sender, CancelEventArgs e)
        {
            try
            {
                UInt16 value = UInt16.Parse(textBoxSourceNpi.Text);
                if (value > 8)
                {
                    throw new FormatException();
                }
                errorProvider.SetError(textBoxSourceNpi, "");
            }
            catch(FormatException)
            {
                errorProvider.SetError(textBoxSourceNpi,"Invalid Npi\n0-Unknown\n1-ISDN(E163/E164)\n2-Data(X.121)\n3-Telex(F.69)\n4-Land Mobile(E.212)\n5-National\n6-Private\n7-ERMES\n8-Internet(IP)");
            }
        }
        private void textBoxDestinationTon_Validating(object sender, CancelEventArgs e)
        {
            try
            {
                UInt16 value = UInt16.Parse(textBoxDestinationTon.Text);
                if (value > 6)
                {
                    throw new FormatException();
                }
                errorProvider.SetError(textBoxDestinationTon, "");
            }
            catch (FormatException)
            {
                errorProvider.SetError(textBoxDestinationTon, "Invalid Ton\n0-Unknown\n1-International\n2-National\n3-Network Specific\n4-Subscriber Number\n5-Alphanumeric\n6-Abbriviated");
            }

        }

        private void textBoxDestinationNpi_Validating(object sender, CancelEventArgs e)
        {
            try
            {
                UInt16 value = UInt16.Parse(textBoxDestinationNpi.Text);
                if (value > 8)
                {
                    throw new FormatException();
                }
                errorProvider.SetError(textBoxDestinationNpi, "");
            }
            catch (FormatException)
            {
                errorProvider.SetError(textBoxDestinationNpi, "Invalid Npi\n0-Unknown\n1-ISDN(E163/E164)\n2-Data(X.121)\n3-Telex(F.69)\n4-Land Mobile(E.212)\n5-National\n6-Private\n7-ERMES\n8-Internet(IP)");
            }

        }
        #endregion
        
        private void buttonSend_Click(object sender, EventArgs e)
        {
            SmppSubmitSm submit = new SmppSubmitSm();
            switch (Int16.Parse(textBoxSourceTon.Text))
            {
                case 0:
                    submit.SourceAddressTon = Pdu.TonType.Unknown;
                    break;
                case 1:
                    submit.SourceAddressTon = Pdu.TonType.International;
                    break;
                case 2:
                    submit.SourceAddressTon = Pdu.TonType.National;
                    break;
                case 3:
                    submit.SourceAddressTon = Pdu.TonType.NetworkSpecific;
                    break;
                case 4:
                    submit.SourceAddressTon = Pdu.TonType.SubsciberNumber;
                    break;
                case 5:
                    submit.SourceAddressTon = Pdu.TonType.Alphanumeric;
                    break;
                case 6:
                    submit.SourceAddressTon = Pdu.TonType.Abbreviated;
                    break;
            }
            switch (Int16.Parse(textBoxSourceNpi.Text))
            {
                case 0:
                    submit.SourceAddressNpi = Pdu.NpiType.Unknown;
                    break;
                case 1:
                    submit.SourceAddressNpi = Pdu.NpiType.ISDN;
                    break;
                case 2:
                    submit.SourceAddressNpi = Pdu.NpiType.Data;
                    break;
                case 3:
                    submit.SourceAddressNpi = Pdu.NpiType.Telex;
                    break;
                case 4:
                    submit.SourceAddressNpi = Pdu.NpiType.LandMobile;
                    break;
                case 5:
                    submit.SourceAddressNpi = Pdu.NpiType.National;
                    break;
                case 6:
                    submit.SourceAddressNpi = Pdu.NpiType.Private;
                    break;
                case 7:
                    submit.SourceAddressNpi = Pdu.NpiType.ERMES;
                    break;
                case 8:
                    submit.SourceAddressNpi = Pdu.NpiType.Internet;
                    break;
            }
            submit.SourceAddress = textBoxSourceAddress.Text;
            switch (Int16.Parse(textBoxDestinationTon.Text))
            {
                case 0:
                    submit.DestinationAddressTon = Pdu.TonType.Unknown;
                    break;
                case 1:
                    submit.DestinationAddressTon = Pdu.TonType.International;
                    break;
                case 2:
                    submit.DestinationAddressTon = Pdu.TonType.National;
                    break;
                case 3:
                    submit.DestinationAddressTon = Pdu.TonType.NetworkSpecific;
                    break;
                case 4:
                    submit.DestinationAddressTon = Pdu.TonType.SubsciberNumber;
                    break;
                case 5:
                    submit.DestinationAddressTon = Pdu.TonType.Alphanumeric;
                    break;
                case 6:
                    submit.DestinationAddressTon = Pdu.TonType.Abbreviated;
                    break;
            }
            switch (Int16.Parse(textBoxSourceNpi.Text))
            {
                case 0:
                    submit.DestinationAddressNpi = Pdu.NpiType.Unknown;
                    break;
                case 1:
                    submit.DestinationAddressNpi = Pdu.NpiType.ISDN;
                    break;
                case 2:
                    submit.DestinationAddressNpi = Pdu.NpiType.Data;
                    break;
                case 3:
                    submit.DestinationAddressNpi = Pdu.NpiType.Telex;
                    break;
                case 4:
                    submit.DestinationAddressNpi = Pdu.NpiType.LandMobile;
                    break;
                case 5:
                    submit.DestinationAddressNpi = Pdu.NpiType.National;
                    break;
                case 6:
                    submit.DestinationAddressNpi = Pdu.NpiType.Private;
                    break;
                case 7:
                    submit.DestinationAddressNpi = Pdu.NpiType.ERMES;
                    break;
                case 8:
                    submit.DestinationAddressNpi = Pdu.NpiType.Internet;
                    break;
            }
            submit.DestinationAddress = textBoxDestinationAddress.Text;
            submit.ShortMessage = textBoxMessage.Text;
            if (checkBoxReplaceIfPresent.Checked == true)
                submit.ReplaceIfPresentFlag = true;
            if (checkBoxRegisterDelivery.Checked == true)
                submit.RegisteredDelivery = Pdu.RegisteredDeliveryType.OnSuccessOrFailure;
            submit.ScheduleDeliveryTime = textBoxScheduleDelivery.Text;
            submit.ValidityPeriod = textBoxValidity.Text;
            switch (config.DCS)
            {
                case 0:
                    submit.DataCoding = Pdu.DataCodingType.SMSCDefault;
                    break;
                case 1:
                    submit.DataCoding = Pdu.DataCodingType.IA5_ASCII;
                    break;
                case 2:
                    submit.DataCoding = Pdu.DataCodingType.OctetUnspecifiedB;
                    break;
                case 3:
                    submit.DataCoding = Pdu.DataCodingType.Latin1;
                    break;
                case 4:
                    submit.DataCoding = Pdu.DataCodingType.OctetUnspecifiedA;
                    break;
                case 5:
                    submit.DataCoding = Pdu.DataCodingType.JIS;
                    break;
                case 6:
                    submit.DataCoding = Pdu.DataCodingType.Cyrillic;
                    break;
                case 7:
                    submit.DataCoding = Pdu.DataCodingType.Latin_Hebrew;
                    break;
                case 8:
                    submit.DataCoding = Pdu.DataCodingType.UCS2;
                    Encoding unicode = Encoding.BigEndianUnicode;
                    byte[] unicodeBytes = unicode.GetBytes(textBoxMessage.Text);
                    submit.ShortMessage = unicodeBytes;
                    break;
                case 9:
                    submit.DataCoding = Pdu.DataCodingType.Pictogram;
                    break;
                case 10:
                    submit.DataCoding = Pdu.DataCodingType.MusicCodes;
                    break;
                case 13:
                    submit.DataCoding = Pdu.DataCodingType.ExtendedKajiJIS;
                    break;
                case 14:
                    submit.DataCoding = Pdu.DataCodingType.KS_C;
                    break;
                default:
                    submit.DataCoding = Pdu.DataCodingType.SMSCDefault;
                    break;
            }
            com.SendPdu(submit);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutDialog about = new AboutDialog();
            if (about.ShowDialog() == DialogResult.OK)
            {
                about.Close();
                about.Dispose();
            }
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OptionsDialog options = new OptionsDialog();
            options.TextBoxServiceType.Text = config.ServiceType;
            options.TextBoxESM.Text = config.ESM.ToString();
            options.TextBoxDCS.Text = config.DCS.ToString();
            options.TextBoxPriority.Text = config.Priority.ToString();
            options.TextBoxEnquireLinkInterval.Text = config.EnquireLinkInterval.ToString();
            if (config.BindType == "Trasceiver")
                options.BindTrasceiver.Checked = true;
            if (config.BindType == "Trasmitter")
                options.BindTrasmitter.Checked = true;
            if (config.BindType == "Receiver")
                options.BindReciver.Checked = true;

            if (options.ShowDialog() == DialogResult.OK) 
            {
                if (options.BindTrasceiver.Checked == true)
                    config.BindType = "Trasceiver";
                if (options.BindTrasmitter.Checked == true)
                    config.BindType = "Trasmitter";
                if (options.BindReciver.Checked == true)
                    config.BindType = "Receiver";
                config.ServiceType = options.TextBoxServiceType.Text;
                config.ESM = UInt16.Parse(options.TextBoxESM.Text);
                config.DCS = UInt16.Parse(options.TextBoxDCS.Text);
                config.Priority = UInt16.Parse(options.TextBoxPriority.Text);
                config.EnquireLinkInterval = UInt16.Parse(options.TextBoxEnquireLinkInterval.Text);
                options.Close();
            }
            options.Dispose();
        }

        
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            SmppCancelSm cancel = new SmppCancelSm();
            try
            {
                Int64 id = Int64.Parse(textBoxMessageId.Text);
                cancel.MessageId = id.ToString("X");
            }
            catch
            {
                ErrorDialog error = new ErrorDialog();
                if (error.ShowDialog() == DialogResult.OK)
                {
                    error.Close();
                    error.Dispose();
                }
            }
            
                     
            com.SendPdu(cancel);
        }

        private void buttonQuery_Click(object sender, EventArgs e)
        {
            SmppQuerySm query = new SmppQuerySm();
            try
            {
                Int64 id = Int64.Parse(textBoxMessageId.Text);
                query.MessageId = id.ToString("X");
            }
            catch
            {
                ErrorDialog error = new ErrorDialog();
                if (error.ShowDialog() == DialogResult.OK)
                {
                    error.Close();
                    error.Dispose();
                }
            }
            switch (Int16.Parse(textBoxSourceTon.Text))
            {
                case 0:
                    query.SourceAddressTon = Pdu.TonType.Unknown;
                    break;
                case 1:
                    query.SourceAddressTon = Pdu.TonType.International;
                    break;
                case 2:
                    query.SourceAddressTon = Pdu.TonType.National;
                    break;
                case 3:
                    query.SourceAddressTon = Pdu.TonType.NetworkSpecific;
                    break;
                case 4:
                    query.SourceAddressTon = Pdu.TonType.SubsciberNumber;
                    break;
                case 5:
                    query.SourceAddressTon = Pdu.TonType.Alphanumeric;
                    break;
                case 6:
                    query.SourceAddressTon = Pdu.TonType.Abbreviated;
                    break;
            }
            switch (Int16.Parse(textBoxSourceNpi.Text))
            {
                case 0:
                    query.SourceAddressNpi = Pdu.NpiType.Unknown;
                    break;
                case 1:
                    query.SourceAddressNpi = Pdu.NpiType.ISDN;
                    break;
                case 2:
                    query.SourceAddressNpi = Pdu.NpiType.Data;
                    break;
                case 3:
                    query.SourceAddressNpi = Pdu.NpiType.Telex;
                    break;
                case 4:
                    query.SourceAddressNpi = Pdu.NpiType.LandMobile;
                    break;
                case 5:
                    query.SourceAddressNpi = Pdu.NpiType.National;
                    break;
                case 6:
                    query.SourceAddressNpi = Pdu.NpiType.Private;
                    break;
                case 7:
                    query.SourceAddressNpi = Pdu.NpiType.ERMES;
                    break;
                case 8:
                    query.SourceAddressNpi = Pdu.NpiType.Internet;
                    break;
            }
                              
            query.SourceAddress = textBoxSourceAddress.Text;

            com.SendPdu(query);
            
        }

        private void saveConfigurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            config.SourceTon = uint.Parse(textBoxSourceTon.Text);
            config.SourceNpi = uint.Parse(textBoxSourceNpi.Text);
            config.SourceAddress = textBoxSourceAddress.Text;
            config.DestinationTon = uint.Parse(textBoxDestinationTon.Text);
            config.DestinationNpi = uint.Parse(textBoxDestinationNpi.Text);
            config.DestinationAddress = textBoxDestinationAddress.Text;
            config.Host = textBoxHost.Text;
            config.Port = uint.Parse(textBoxPort.Text);
            config.SystemId = textBoxSystemId.Text;
            config.Password = textBoxPassword.Text;
            Configuration.Serialize(@configPath, config);
        }

        private void SmppClient_FormClosing(object sender, FormClosingEventArgs e)
        {
            com.Unbind();
        }

           
       
    }
}