using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Packet;
using SMPP.Utility;


namespace SMPP.Packet.Request
{
    public abstract class MessageLcd2:MessageLcd3
    {
        public const int SHORT_MESSAGE_LIMIT = 160;

        #region fields
        private SmppVersionType protocolId = Pdu.SmppVersionType.Version3_4;
        private PriorityType priorityFlag = Pdu.PriorityType.Level1;
        private string scheduleDeliveryTime = string.Empty;
        private string validityPeriod = string.Empty;
        private bool replaceIfPresentFlag = false;
        private byte smDefaultMessageId = 0;
        private object shortMessage = null;

        protected byte smLength = 0;
        #endregion

        #region properties 
        public SmppVersionType ProtocolId
        {
            get
            {
                return this.protocolId;
            }
            set
            {
                this.protocolId = value;
            }
        }

        public PriorityType PriorityFlag
        {
            get
            {
                return this.priorityFlag;
            }
            set
            {
                this.priorityFlag = value;
            }
        }

        public string ScheduleDeliveryTime
        {
            get
            {
                return this.scheduleDeliveryTime;
            }
            set
            {
                if (value != null && value != string.Empty)
                {
                    if (value.Length == DATE_TIME_LENGTH)
                    {
                        this.scheduleDeliveryTime = value;
                    }
                    else
                    {
                        throw new ArgumentException("Scheduler delivery time not in correct format.");
                    }
                }
                else
                {
                    this.scheduleDeliveryTime = string.Empty;
                }
            }
        }

        public string ValidityPeriod
        {
            get
            {
                return this.validityPeriod;
            }
            set
            {
                if (value != null && value != string.Empty)
                {
                    if (value.Length == DATE_TIME_LENGTH)
                    {
                        this.validityPeriod = value;
                    }
                    else
                    {
                        throw new ArgumentException("Validity format nit in correct format.");
                    }
                }
                else
                {
                    this.validityPeriod = string.Empty;
                }
            }
        }

        public bool ReplaceIfPresentFlag
        {
            get
            {
                return this.replaceIfPresentFlag;
            }
            set
            {
                this.replaceIfPresentFlag = value;
            }
        }

        public byte SmDefaultMessageId
        {
            get
            {
                return this.smDefaultMessageId;
            }
            set
            {
                this.smDefaultMessageId = value;
            }
        }

        public object ShortMessage
        {
            get
            {
                return this.shortMessage;
            }
            set
            {
                this.shortMessage = value;
            }
        }

        public byte SmLength
        {
            get
            {
                return this.smLength;
            }
        }
        #endregion

        #region constructors
        public MessageLcd2(): base()
        { }
        public MessageLcd2(byte[] incomingBytes): base(incomingBytes)
        { }
        #endregion

        protected ArrayList GetBytesAfterDesination()
        {
            ArrayList pdu = new ArrayList();
            pdu.Add(EsmClass);
            pdu.Add((byte)ProtocolId);
            pdu.Add((byte)PriorityFlag);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ScheduleDeliveryTime)));
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ValidityPeriod)));
            pdu.Add((byte)RegisteredDelivery);
            if (ReplaceIfPresentFlag == true)
            {
                pdu.Add((byte)0x01);
            }
            else
            {
                pdu.Add((byte)0x00);
            }
            pdu.Add((byte)DataCoding);
            pdu.Add(SmDefaultMessageId);
            this.smLength = PduUtil.InsertShortMessage(pdu, ShortMessage);
            pdu.TrimToSize();

            return pdu;
        }

    }
}
