using System;
using System.Collections.Generic;
using System.Text;
using SMPP.Packet;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public abstract class MessageLcd3:MessageLcd4
    {

        #region fields
        private string serviceType = string.Empty;
        private byte esmClass = 0;
        private DataCodingType dataCoding = Pdu.DataCodingType.SMSCDefault;
        #endregion

        #region constants 
        private const int SAR_MIN = 1;
        private const int SAR_MAX = 255;
        #endregion

        #region properties

        public string ServiceType
        {
            get
            {
                return this.serviceType;
            }
            set
            {
                if (value != null)
                {
                    if (value.Length <= SERVICE_TYPE_LENGTH)
                    {
                        this.serviceType = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Service type must be " + SERVICE_TYPE_LENGTH
                            + " 5 characters or less.");
                    }
                }
                else
                {
                    this.serviceType = string.Empty;
                }
            }
        }
        public byte EsmClass
        {
            get 
            {
                return this.esmClass;
            }
            set
            {
                this.esmClass = value;
            }
        }

        public DataCodingType DataCoding
        {
            get
            {
                return this.dataCoding;
            }
            set
            {
                this.dataCoding = value;
            }
        }
        #endregion

        #region optional params

        public UInt16 SourcePort
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.source_port);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {
                    SetHostOrderValueIntoTlv((UInt16)Pdu.OptionalParamCodes.source_port, value);
                }
                else
                {
                    throw new ArgumentException("source_port value  too large.");
                }
            }
        }

        public AddressSubunitType SourceAddressSubunit
        {
            get
            {
                return (AddressSubunitType)GetOptionalParamBytes((ushort)OptionalParamCodes.source_addr_subunit)[0];
            }
            set
            {
                SetOptionalParamBytes(
                    (ushort)Pdu.OptionalParamCodes.source_addr_subunit,new Byte[] {(byte)value});
            }
        }
        public UInt16 DestinationPort
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.destination_port);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {
                    SetHostOrderValueIntoTlv((UInt16)Pdu.OptionalParamCodes.destination_port, value);
                }
                else
                {
                    throw new ArgumentException("destination_port value too large.");
                }
            }
        }

        public UInt16 SarMsgRefNumber
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.sar_msg_ref_num);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {
                    SetHostOrderValueIntoTlv((UInt16)Pdu.OptionalParamCodes.sar_msg_ref_num, value);
                }
                else
                {
                    throw new ArgumentException("sar_msg_ref_number value too large.");
                }
            }
        }

        public byte SarTotalSegments
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.sar_total_segments)[0];
            }
            set
            {
                if (value >= SAR_MIN && value <= SAR_MAX)
                {
                    SetOptionalParamBytes(
                        (ushort)Pdu.OptionalParamCodes.sar_total_segments, new Byte[] { (byte)value });
                }
                else
                {
                    throw new ArgumentException("sar_total_segmentsmust be >= " + SAR_MIN + "and<=" + SAR_MAX);
                }
            }
        }
        
        public byte SarSegmentSeqnum
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.sar_segment_seqnum)[0];
            }
            set
            {
                if (value >= SAR_MIN && value <= SAR_MAX)
                {
                    SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.sar_segment_seqnum, new Byte[] { (byte)value });
                }
                else
                {
                    throw new ArgumentException(" sar_segmenr_seqnum must be " + SAR_MIN + "and <=" + SAR_MAX);
                }

            }
        }

        public PayloadTypeType PayloadType
        {
            get
            {
                return (PayloadTypeType)GetOptionalParamBytes((ushort)OptionalParamCodes.payload_type)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.payload_type, new Byte[] { (byte)value });
            }
        }

        public object MessagePayload
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.message_payload)[0];
            }
            set
            {
                PduUtil.SetMessagePayload(this, value);
            }
        }

        public PrivacyType PrivaceIndicator
        {
            get
            {
                return (PrivacyType)GetOptionalParamBytes((ushort)OptionalParamCodes.privacy_indicator)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.privacy_indicator, new Byte[] { (byte)value });
            }
        }

        public UInt16 UserMessageReference
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.user_message_reference);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {

                    SetHostOrderValueIntoTlv((ushort)Pdu.OptionalParamCodes.user_message_reference, value);
                }
                else
                {
                    throw new ArgumentException("user_message_reference too large");
                }
            }
        }

        public byte MsgMsWaitFacilities
        {
            get
            {
                return GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.ms_msg_wait_failities)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.ms_msg_wait_failities, new Byte[] { (byte)value });
            }
        }

        public MsValidityType MsValidity
        {
            get
            {
                return (MsValidityType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.ms_validity)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.ms_validity, new Byte[] { (byte)value });
            }
        }

        public UInt16 SmsSignal
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.sms_signal);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {

                    SetHostOrderValueIntoTlv((ushort)Pdu.OptionalParamCodes.sms_signal, value);
                }
                else
                {
                    throw new ArgumentException("sms_signal value too large");
                }
            }
        }

        public AddressSubunitType DestinationAddrSubunit
        {
            get
            {
                return (AddressSubunitType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.dest_addr_subunit)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.dest_addr_subunit, new Byte[] { (byte)value });
            }
        }

        public byte AlertOnMsgDelivery
        {
            get
            {
                try
                {
                    byte[] bytes = GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.alert_on_message_delivery);
                    return 1;
                }
                catch (ApplicationException)
                {
                    return 0;
                }
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.alert_on_message_delivery, new Byte[0]);
            }
        }

        public LanguageType LanguageIndicator
        {
            get
            {
                return (LanguageType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.language_indicator)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.language_indicator, new Byte[] { (byte)value });
            }
        }

        public DisplayTimeType DisplayTime
        {
            get
            {
                return (DisplayTimeType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.display_time)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.display_time, new Byte[] { (byte)value });
            }
        }

        public string CallbackNum
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.callback_num);
            }
            set
            {
                PduUtil.SetCallbackNum(this, value);
            }
        }

        public byte CallbackNumPressInd
        {
            get
            {
                return GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.callback_num_pres_ind)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.callback_num_pres_ind, new Byte[] { (byte)value });
            }
        }

        public string CallbackNumAtag
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.callback_num_atag);
            }
            set
            {
                const int CALL_BACK_ATAG = 65;

                if (value.Length <= CALL_BACK_ATAG)
                {
                    SetOptionalParamString((ushort)Pdu.OptionalParamCodes.callback_num_atag, value);
                }
                else
                {
                    throw new ArgumentException("Callback number atag must be <=" + CALL_BACK_ATAG + ".");
                }
            }
        }

        public string SourceSubAddress
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.source_subaddress);
            }
            set
            {
                PduUtil.SetDestSubaddress(this, value);
            }
        }

        public string DestinationSbAddress
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.dest_subaddress);
            }
            set
            {
                SetOptionalParamString((ushort)Pdu.OptionalParamCodes.dest_subaddress, value);
            }
        }

        #endregion

        #region constructors

        protected MessageLcd3(): base()
        { }

        protected MessageLcd3(byte[] incomingBytes): base(incomingBytes)
        { }

        #endregion
        
    }
}



