using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppAlertNotification:Pdu
    {
        #region fields
        private TonType sourceAddressTon = TonType.International;
        private NpiType sourceAddressNpi = NpiType.ISDN;
        private string sourceAddress = string.Empty;
        private TonType esmeAddressTon = TonType.International;
        private NpiType esmeAddressNpi = NpiType.ISDN;
        private string emseAddress = string.Empty;
        #endregion

        #region enumeration
        public enum AvailabilityStatusType : byte
        {
            Available = 0x00,
            Denied = 0x01,
            Unavailable = 0x02
        }
        #endregion


        #region properties
        public TonType SourceAddressTon
        {
            get { return this.sourceAddressTon; }
            set { this.sourceAddressTon = value; }
        }
        public NpiType SourceAddressNpi
        {
            get { return this.sourceAddressNpi; }
            set { this.sourceAddressNpi = value; }
        }
        public string SourceAddress
        {
            get { return this.sourceAddress; }
            set
            {
                this.sourceAddress = (value == null) ? string.Empty : value;
            }
        }

        public TonType EsmeAddressTon
        {
            get { return this.esmeAddressTon; }
            set { this.esmeAddressTon = value; }
        }
        public NpiType EsmeAddressNpi
        {
            get { return this.esmeAddressNpi; }
            set { this.esmeAddressNpi = value; }
        }
        public string EsmeAddress
        {
            get { return this.emseAddress; }
            set
            {
                this.emseAddress = (value == null) ? string.Empty : value;
            }
        }
        public AvailabilityStatusType MSAvailabilityStatus
        {
            get
            {
                return (AvailabilityStatusType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.ms_availability_status)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.ms_availability_status, new Byte[] { (byte)value });
            }
        }
        #endregion

        #region constructors
        public SmppAlertNotification(): base()
        { }
        public SmppAlertNotification(byte[] incomigBytes): base(incomigBytes)
        { }
        #endregion

        protected override void DecodeSmscResponse()
        {
            byte[] remainder = BytesAfterHeader;
            SourceAddressTon = (TonType)remainder[0];
            SourceAddressNpi = (NpiType)remainder[1];
            SourceAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);
            EsmeAddressTon = (TonType)remainder[0];
            EsmeAddressNpi = (NpiType)remainder[1];
            EsmeAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);

            TranslateTlvDataIntoTable(remainder);
        }

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = CommandIdType.alert_notification;
        }

        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.Add((byte)SourceAddressTon);
            pdu.Add((byte)SourceAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SourceAddress)));
            pdu.Add((byte)EsmeAddressTon);
            pdu.Add((byte)EsmeAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(EsmeAddress)));

            PacketBytes = EncodePduForTrasmission(pdu);
        }



    }
}
