using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppBind : Pdu
    {
        #region fields
        private string systemId = string.Empty;
        private string password = string.Empty;
        private string systemType = string.Empty;
        private string addressRange = string.Empty;
        private SmppVersionType interfaceVersion = Pdu.SmppVersionType.Version3_4;
        private TonType addressTon = Pdu.TonType.International;
        private NpiType addressNpi = Pdu.NpiType.ISDN;
        private BindingType bindType = BindingType.BindAsTransceiver;
        #endregion

        #region constants
        private const int ID_LENGTH = 15;
        private const int PASS_LENGTH = 8;
        private const int TYPE_LENGTH = 12;
        private const int RANGE_LENGTH = 40;
        #endregion

        #region properties
        public BindingType BindType
        {
            get{return this.bindType;}
            set
            {
                switch(value)
                {
                    case BindingType.BindAsReceiver:
                        {
                            CommandID = CommandIdType.bind_receiver;
                            break;
                        }
                    case BindingType.BindAsTransceiver:
                        {
                            CommandID = CommandIdType.bind_transceiver;
                             break;
                        }
                    case BindingType.BindAsTrasmitter:
                        {
                            CommandID = CommandIdType.bind_trasmitter;
                            break;
                        }
                    default:
                        {
                        CommandID = CommandIdType.bind_trasmitter;
                        break;
                        }

                }
                this.bindType = value;
            }
        }

        public string SystemId
        {
            get
            {
                return this.systemId;
            }
            set
            {
                if (value != null)
                {
                    if (value.Length < ID_LENGTH)
                        this.systemId = value;
                    else
                    {
                        throw new ArgumentOutOfRangeException("System ID must be <=" + ID_LENGTH + "characters.");
                    }
                }
                else this.systemId = value;
            }
        }

        public string Password
        {
            get
            {
                return this.password;
            }
            set
            {
                if(value != null)
                {
                    if(value.Length <= PASS_LENGTH)
                    {
                        this.password = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Password must be <="+PASS_LENGTH+"characters.");
                    }
                }
                else
                {
                    this.password = string.Empty;
                }
            }
        }

        public string SystemType
        {
            get
            {
                return this.systemType;
            }
            set
            {
                if(value != null)
                {
                    if(value.Length < TYPE_LENGTH )
                    {
                        this.systemType = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("system type must be <= "+ TYPE_LENGTH + "characters.");
                    }
                }
                else
                {
                    this.systemType = string.Empty;
                }
            }
        }

        public string AddressRange
        {
            get { return this.addressRange; }
            set
            {
                if (value != null)
                {
                    if (value.Length <= RANGE_LENGTH)
                    {
                        this.addressRange = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Address range must be <=" + RANGE_LENGTH + "charactres.");
                    }
                }
                else
                {
                    this.addressRange = string.Empty;
                }
            }
        }


        public SmppVersionType InterfaceVersion
        {
            get { return this.interfaceVersion; }
            set { this.interfaceVersion = value; }
        }

        public TonType AddressTon
        {
            get { return this.addressTon; }
            set { this.addressTon = value; }
        }
        public NpiType AddressNpi
        {
            get { return this.addressNpi; }
            set { this.addressNpi = value; }
        }

        #endregion

        #region enumeration
        public enum BindingType : uint
        {
            BindAsReceiver = 1,
            BindAsTrasmitter = 2,
            BindAsTransceiver = 9
        }
        #endregion

        #region constructors
        public SmppBind(): base()
        { }
        public SmppBind(byte[] incomingBytes): base(incomingBytes)
        { }
        #endregion

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            BindType = BindingType.BindAsTransceiver;
        }

        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SystemId)));
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(Password)));
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SystemType)));
            pdu.Add((byte)InterfaceVersion);
            pdu.Add((byte)AddressTon);
            pdu.Add((byte)AddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(AddressRange)));

            PacketBytes = EncodePduForTrasmission(pdu);

        }

        protected override void DecodeSmscResponse()
        {
            byte[] remainder = BytesAfterHeader;
            BindType = (SmppBind.BindingType)CommandID;
            SystemId = SmppStringUtil.GetCStringFromBody(ref remainder);
            Password = SmppStringUtil.GetCStringFromBody(ref remainder);
            SystemType = SmppStringUtil.GetCStringFromBody(ref remainder);
            InterfaceVersion = (SmppVersionType)remainder[0];
            AddressTon = (TonType)remainder[1];
            AddressNpi = (NpiType)remainder[2];
            AddressRange = SmppStringUtil.GetCStringFromBody(ref remainder, 3);
            TranslateTlvDataIntoTable(remainder);

        }





















    }
}
