using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Packet;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppCancelSm:MessageLcd6
    {
        #region fields
        private string messageId = string.Empty;
        private string serviceType = string.Empty;
        private TonType destinationAddressTon = Pdu.TonType.International;
        private NpiType destinationAddressNpi = Pdu.NpiType.ISDN;
        private string destinationAddress = string.Empty;
        #endregion

        #region properties
        public string MessageId
        {
            get { return this.messageId; }
            set
            {
                if (value != null)
                {
                    if (value.Length <= MSG_LENGTH)
                    {
                        this.messageId = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Message Id must be" + MSG_LENGTH + " characters.");
                    }
                }
                else
                {
                    this.messageId = string.Empty;
                }
            }
        }
        public string ServiceType
        {
            get { return this.serviceType; }
            set
            {
                if (value != null)
                {
                    if (value.Length <= SERVICE_TYPE_LENGTH)
                    {
                        this.serviceType = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Service type must be " + SERVICE_TYPE_LENGTH + "characters.");
                    }
                }
                else
                {
                    this.serviceType = string.Empty;
                }
            }
        }

        public TonType DestinationAddressTon
        {
            get
            {
                return this.destinationAddressTon;
            }
            set
            {
                this.destinationAddressTon = value;
            }
        }

        public NpiType DestinationAddressNpi
        {
            get
            {
                return this.destinationAddressNpi;
            }
            set
            {
                this.destinationAddressNpi = value;
            }
        }

        public string DestinationAddress
        {
            get
            {
                return this.destinationAddress;
            }
            set
            {
                if (value != null)
                {
                    if (value.Length <= ADDRESS_LENGTH)
                    {
                        this.destinationAddress = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Destination address must be" + ADDRESS_LENGTH + "characters.");
                    }
                }
                else
                {
                    this.destinationAddress = string.Empty;
                }
            }
        }
        #endregion

        #region constructors
        public SmppCancelSm(): base()
        { }

        public SmppCancelSm(byte[] incomingBytes): base(incomingBytes)
        { }
        #endregion

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = Pdu.CommandIdType.cancel_sm;

        }

        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ServiceType)));
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(MessageId)));
            pdu.Add((byte)SourceAddressTon);
            pdu.Add((byte)SourceAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SourceAddress)));
            pdu.Add((byte)destinationAddressTon);
            pdu.Add((byte)destinationAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(DestinationAddress)));

            PacketBytes = EncodePduForTrasmission(pdu);
        }

        protected override void DecodeSmscResponse()
        {
            byte[] remainder = BytesAfterHeader;
            ServiceType = SmppStringUtil.GetCStringFromBody(ref remainder);
            MessageId = SmppStringUtil.GetCStringFromBody(ref remainder);
            SourceAddressTon = (TonType)remainder[0];
            SourceAddressNpi = (NpiType)remainder[1];
            SourceAddress = SmppStringUtil.GetCStringFromBody(ref remainder,2);
            DestinationAddressTon = (TonType)remainder[0];
            DestinationAddressNpi = (NpiType)remainder[1];
            DestinationAddress = SmppStringUtil.GetCStringFromBody(ref remainder,2);

            TranslateTlvDataIntoTable(remainder);
        }



    }
}
