using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppDataSm : MessageLcd3
    {
        #region fileds
        private TonType destinationAddressTon = Pdu.TonType.International;
        private NpiType destinationAddressNpi = Pdu.NpiType.ISDN;
        private string destinationAddress = string.Empty;
        #endregion

        #region properties
        public TonType DestinationAddressTon
        {
            get { return this.destinationAddressTon; }
            set { this.destinationAddressTon = value; }
        }
        public NpiType DestinationAddressNpi
        {
            get { return this.destinationAddressNpi; }
            set { this.destinationAddressNpi = value; }
        }
        public string DestinationAddress
        {
            get
            {
                return this.destinationAddress;
            }
            set
            {
                if (value != null)
                {
                    if (value.Length <= ADDRESS_LENGTH)
                    {
                        this.destinationAddress = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Destination Address must be <=" + ADDRESS_LENGTH + "characters.");
                    }
                }
                else
                {
                    this.destinationAddress = string.Empty;
                }
            }
        }
        #endregion

        #region optional parameters
        public NetworkType SourceNetworkType
        {
            get
            {
                return (NetworkType)(GetOptionalParamBytes((ushort)OptionalParamCodes.source_network_type)[0]);
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.source_network_type, new Byte[] { (byte)value });
            }
        }

        public BearerType SourecBearerType
        {
            get
            {
                return (BearerType)(GetOptionalParamBytes((ushort)OptionalParamCodes.source_bearer_type)[0]);
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.source_bearer_type, new Byte[] { (byte)value });
            }
        }

        public UInt16 SourceTelematicsId
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.source_telematics_id);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {
                    SetOptionalParamBytes((UInt16)Pdu.OptionalParamCodes.source_telematics_id, BitConverter.GetBytes(UnsignedNumConverter.SwapByteOrdering(value)));
                }
                else
                {
                    throw new ArgumentException("source_telematics_id value too large");
                }
            }
        }

        public NetworkType DestinationNetworkType
        {
            get
            {
                return (NetworkType)GetOptionalParamBytes((ushort)OptionalParamCodes.dest_network_type)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.dest_network_type, new Byte[] { (byte)value });
            }
        }

        public BearerType DestBearerType
        {
            get
            {
                return (BearerType)GetOptionalParamBytes((ushort)OptionalParamCodes.dest_bearer_type)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.dest_bearer_type, new Byte[] { (byte)value });
            }
        }

        public UInt16 DestTelematicsId
        {
            get
            {
                return GetHostOrderUInt16FromTlv((ushort)Pdu.OptionalParamCodes.dest_telematics_id);
            }
            set
            {
                if (value < UInt16.MaxValue)
                {
                    SetOptionalParamBytes((UInt16)Pdu.OptionalParamCodes.dest_telematics_id, BitConverter.GetBytes(UnsignedNumConverter.SwapByteOrdering(value)));
                }
                else
                {
                    throw new ArgumentException("dest_telematics_id value too large");
                }
            }
        }

        public bool MoreMessagesToSend
        {
            get
            {
                byte sendMore = GetOptionalParamBytes((ushort)OptionalParamCodes.more_messages_to_send)[0];
                return (sendMore == 0) ? false : true;
            }
            set
            {
                byte sendMore;
                if (value == false)
                {
                    sendMore = (byte)0x00;
                }
                else
                {
                    sendMore = (byte)0x01;
                }
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.more_messages_to_send, new Byte[] { sendMore });
            }
        }

        public UInt32 QoSTimeToLive
        {
            get
            {
                return GetHostOrderUInt32FromTlv((ushort)Pdu.OptionalParamCodes.qos_time_to_live);
            }
            set
            {
                if (value < UInt32.MaxValue)
                {
                    SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.qos_time_to_live, BitConverter.GetBytes(UnsignedNumConverter.SwapByteOrdering(value)));
                }
                else
                {
                    throw new ArgumentException("qos_time_to_live value too large");
                }
            }
        }
        public DpfResultType SetDpf
        {
            get
            {
                return (DpfResultType)GetOptionalParamBytes((ushort)OptionalParamCodes.set_dpf)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.set_dpf, new Byte[] { (byte)value });
            }

        }

        public string RecepientMessageId
        {
            get
            {
                return GetOptionalParamString((ushort)OptionalParamCodes.receipted_message_id);
            }
            set
            {
                PduUtil.SetReceiptedMessageId(this, value);
            }
        }

        public MessageStateType MessageState
        {
            get
            {
                return (MessageStateType)GetOptionalParamBytes((ushort)OptionalParamCodes.message_state)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.message_state, new Byte[] { (byte)value });
            }
        }

        public string NetworkErrorCode
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.network_error_code);
            }
            set
            {
                PduUtil.SetNetworkErrorCode(this, value);
            }
        }

        public byte UserResponseCode
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.user_response_code)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.user_response_code, new Byte[] { (byte)value });
            }
        }

        public byte NumberOfMessages
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.number_of_messages)[0];
            }
            set
            {
                if (value <= 99)
                {
                    SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.number_of_messages, new Byte[] { value });
                }
                else
                {
                    throw new ArgumentException("number_of_messages must be between 0 and 99");
                }

            }
        }

        public ItsReplyTypeType ItsReplyType
        {
            get
            {
                return (ItsReplyTypeType)GetOptionalParamBytes((ushort)OptionalParamCodes.its_replay_type)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.its_replay_type, new Byte[] { (byte)value });
            }
        }

        public string SessionControlInformationa
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.its_session_info);
            }
            set
            {
                PduUtil.SetItsSessionInfo(this, value);
            }
        }

        #endregion

        #region constructors
        public SmppDataSm(): base()
        { }

        public SmppDataSm(byte[] incomigBytes): base(incomigBytes)
        { }
        #endregion

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = CommandIdType.data_sm;
        }

        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ServiceType)));
            pdu.Add((byte)SourceAddressTon);
            pdu.Add((byte)SourceAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SourceAddress)));
            pdu.Add((byte)DestinationAddressTon);
            pdu.Add((byte)destinationAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(DestinationAddress)));
            pdu.Add(EsmClass);
            pdu.Add((byte)RegisteredDelivery);
            pdu.Add((byte)DataCoding);

            PacketBytes = EncodePduForTrasmission(pdu);
        }

        protected override void DecodeSmscResponse()
        {
            byte[] remainder = BytesAfterHeader;
            ServiceType = SmppStringUtil.GetCStringFromBody(ref remainder);
            SourceAddressTon = (TonType)remainder[0];
            SourceAddressNpi = (NpiType)remainder[1];
            SourceAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);
            DestinationAddressTon = (TonType)remainder[0];
            DestinationAddressNpi = (NpiType)remainder[1];
            DestinationAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);
            EsmClass = remainder[0];
            RegisteredDelivery = (RegisteredDeliveryType)remainder[1];
            DataCoding = (DataCodingType)remainder[2];

            TranslateTlvDataIntoTable(remainder,3);
         

        }
  
    }
}
