using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppReplaceSm : MessageLcd4
    {
        #region fields
        private string messageId = string.Empty;
        private string scheduleDeliveryTime = string.Empty;
        private string valididtyPeriod = string.Empty;
        private byte smDefaultMessageId = 0;
        private object shortMessage = null;
        private byte smLength = 0;
        #endregion

        #region properties
        public string MessageId
        {
            get { return this.messageId; }
            set
            {
                if (value != null)
                {
                    if (value.Length <= MSG_LENGTH)
                    {
                        this.messageId = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Message Id muste be" + MSG_LENGTH + "characters.");
                    }
                }
                else
                {
                    this.messageId = string.Empty;
                }
            }
        }

        public string ScheduleDeliveryTime
        {
            get{return this.scheduleDeliveryTime;}
            set
            {
                if(value!=null && value !=string.Empty)
                {
                    if(value.Length == DATE_TIME_LENGTH)
                    {
                        this.scheduleDeliveryTime = value;
                    }
                    else
                    {
                        throw new ArgumentException("Scheduled delivery time not in correct format.");
                    }
                }
                else
                {
                    this.scheduleDeliveryTime = string.Empty;
                }
            }
        }

        public string ValidityPeriod
        {
            get { return this.valididtyPeriod; }
            set
            {
                if (value != null && value != string.Empty)
                {
                    if (value.Length == DATE_TIME_LENGTH)
                    {
                        this.valididtyPeriod = value;
                    }
                    else
                    {
                        throw new ArgumentException("Validity period not in correct format.");
                    }
                }
                else
                {
                    this.valididtyPeriod = string.Empty;
                }
            }
        }

        public byte SmDefaultMessageId
        {
            get{return this.smDefaultMessageId;}
            set{this.smDefaultMessageId = value;}
        }

        public object ShortMessage
        {
            get
            {
                return this.shortMessage;
            }
            set
            {
                this.shortMessage = value;
            }
        }
        public byte SmLenght
        {
            get
            {
                return this.smLength;
            }
        }
        #endregion

        #region constructors
        public SmppReplaceSm(): base()
        { }
        public SmppReplaceSm(byte[] incomigBytes):base(incomigBytes)
        { }
        #endregion

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = CommandIdType.replace_sm;
        }

        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(MessageId)));
            pdu.Add((byte)SourceAddressTon);
            pdu.Add((byte)SourceAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SourceAddress)));
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ScheduleDeliveryTime)));
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ValidityPeriod)));
            pdu.Add((byte)RegisteredDelivery);
            pdu.Add(SmDefaultMessageId);
            this.smLength = PduUtil.InsertShortMessage(pdu, ShortMessage);
            PacketBytes = EncodePduForTrasmission(pdu);
        }

        protected override void DecodeSmscResponse()
        {
            byte[] remainder = BytesAfterHeader;
            MessageId = SmppStringUtil.GetCStringFromBody(ref remainder);
            SourceAddressTon = (TonType)remainder[0];
            SourceAddressNpi = (NpiType)remainder[1];
            ScheduleDeliveryTime = SmppStringUtil.GetCStringFromBody(ref remainder);
            ValidityPeriod = SmppStringUtil.GetCStringFromBody(ref remainder);
            RegisteredDelivery = (RegisteredDeliveryType)remainder[0];
            SmDefaultMessageId = remainder[1];
            this.smLength = remainder[2];
            ShortMessage = SmppStringUtil.GetCStringFromBody(ref remainder, 3, 3 + SmLenght);

            TranslateTlvDataIntoTable(remainder);
        }
    }

}
