using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppSubmitMulti:MessageLcd2
    {
        #region fields
        private byte numberOfDests = 0;
        private DestinationAddress[] destinationAddresses = new DestinationAddress[0];
        private const int MAX_DESTS = 254;
        #endregion

        #region properties
        public byte NumberOfDestinations
        {
            get{return this.numberOfDests;}
        }
        public DestinationAddress[] DestinationAddresses
        {
            get
            {
                return (DestinationAddress[])this.destinationAddresses.Clone();
            }
            set
            {
                this.destinationAddresses = (value == null) ? new DestinationAddress[0] : value;
                this.numberOfDests = (byte)this.destinationAddresses.Length;
            }
        }
        #endregion

        #region constructors
        public SmppSubmitMulti() : base()
        { }
        public SmppSubmitMulti(byte[] incomigBytes): base(incomigBytes)
        { }
        #endregion

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = CommandIdType.submit_multi;

        }


        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ServiceType)));
            pdu.Add((byte)SourceAddressTon);
            pdu.Add((byte)SourceAddressNpi);
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SourceAddress)));
            pdu.Add(NumberOfDestinations);

            foreach (DestinationAddress address in this.DestinationAddresses)
            {
                if (!address.IsDistributionList)
                {
                    ArrayList sme = new ArrayList();
                    pdu.Add((byte)0x01);
                    pdu.Add((byte)address.DestinationAddressTon);
                    pdu.Add((byte)address.DestinationAddressNpi);
                    pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(address.DestAddress)));
                }
                else
                {
                    ArrayList dln = new ArrayList();
                    pdu.Add((byte)0x02);
                    pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(address.DistributionList)));
                }
            }
            pdu.AddRange(base.GetBytesAfterDesination());

            PacketBytes = EncodePduForTrasmission(pdu);
        }

        protected override void  DecodeSmscResponse()
        {
            byte[] remainder = BytesAfterHeader;

            ServiceType = SmppStringUtil.GetCStringFromBody(ref remainder);
            SourceAddressTon = (TonType)remainder[0];
            SourceAddressNpi = (NpiType)remainder[1];
            SourceAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);

            if(remainder.Length > 0)
            {
                this.numberOfDests = remainder[0];
                DestinationAddresses = new DestinationAddress[NumberOfDestinations];

                long length = remainder.Length - 1;
                byte[] newRemainder = new byte[length];
                Array.Copy(remainder,1,newRemainder,0,length);
                remainder = newRemainder;
                newRemainder = null;

                for(int i = 0; i < this.destinationAddresses.Length; i++)
                {
                    this.destinationAddresses[i] = new DestinationAddress(ref remainder);
                }
            }

            EsmClass = remainder[0];
            ProtocolId=(SmppVersionType)remainder[1];
            PriorityFlag=(PriorityType)remainder[2];
            ScheduleDeliveryTime = SmppStringUtil.GetCStringFromBody(ref remainder,3);
            ValidityPeriod = SmppStringUtil.GetCStringFromBody(ref remainder);
            RegisteredDelivery = (RegisteredDeliveryType)remainder[0];
            ReplaceIfPresentFlag = (remainder[1] == 0)? false : true;
            DataCoding = (DataCodingType)remainder[2];
            SmDefaultMessageId = remainder[3];
            this.smLength = remainder[4];
            ShortMessage = SmppStringUtil.GetCStringFromBody(ref remainder, 5, 5 + smLength);


            TranslateTlvDataIntoTable(remainder);
        }

    } 



    
}
