using System;
using System.Collections.Generic;
using System.Text;
using SMPP.Packet;
using System.Collections;
using SMPP.Utility;

namespace SMPP.Packet.Request
{
    public class SmppSubmitSm:MessageLcd2
    {
        #region
        private TonType destinationAddressTon;
        private NpiType destinationAddressNpi;
        private string destinationAddress;
        #endregion

        #region properties
        public TonType DestinationAddressTon
        {
            get { return this.destinationAddressTon; }
            set { this.destinationAddressTon = value; }
        }
        public NpiType DestinationAddressNpi
        {
            get { return this.destinationAddressNpi; }
            set { this.destinationAddressNpi = value; }
        }
        public string DestinationAddress
        {
            get { return this.destinationAddress; }
            set
            {
                if (value != null)
                {
                    if (value.ToString().Length <= ADDRESS_LENGTH)
                    {
                        this.destinationAddress = value;
                    }
                    else
                    {
                        throw new ArgumentOutOfRangeException("Destination address too long (must be <=" + ADDRESS_LENGTH + "characters.");
                    }
                }
            }
        }

        #endregion

        #region optinal parameters
        public bool MoreMessagesToSend
        {
            get
            {
                byte[] bytes = GetOptionalParamBytes((ushort)OptionalParamCodes.more_messages_to_send);
                return (bytes[0] == 0) ? false : true;
            }
            set
            {
                byte sendMore;
                if (value == false)
                {
                    sendMore = (byte)0x00;
                }
                else
                {
                    sendMore = (byte)0x01;
                }
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.more_messages_to_send, new Byte[] { sendMore });
            }
        }

        public byte UserResponseCode
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.user_response_code)[0];
            }
            set
            {
                SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.user_response_code, new Byte[] { value });
            }
        }

        public byte NumberOfMessages
        {
            get
            {
                return GetOptionalParamBytes((ushort)OptionalParamCodes.number_of_messages)[0];
            }
            set
            {
                const int MAX_NUM_MSG = 99;

                if(value <= MAX_NUM_MSG)
                {
                    SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.number_of_messages,new Byte[] {value});
                }
                else
                {
                   throw new ArgumentException("number_of_messages must be between 0 and "+MAX_NUM_MSG+".");
                }
            }
        }

            public ItsReplyTypeType ItsReplyType
            {
                get
                {
                    return (ItsReplyTypeType)GetOptionalParamBytes((ushort)OptionalParamCodes.its_replay_type)[0];
                }
                set
                {
                    SetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.its_replay_type, new Byte[] { (byte)value });
                }
            }

            public string ItsSessionInfo
            {
                get
                {
                    return GetOptionalParamString((ushort)OptionalParamCodes.its_session_info);
                }
                set
                {
                    PduUtil.SetItsSessionInfo(this, value);
                }
            }

            public string UssdServiceOp
            {
                get
                {
                    return GetOptionalParamString((ushort)OptionalParamCodes.ussd_service_op);
                }
                set
                {
                    if (value == null)
                    {
                        SetOptionalParamString((ushort)Pdu.OptionalParamCodes.ussd_service_op, String.Empty);
                    }
                    else if (value.Length == 1)
                    {
                        SetOptionalParamString((ushort)Pdu.OptionalParamCodes.ussd_service_op, value);
                    }
                    else
                    {
                        throw new ArgumentException("ussd_service_op must have lenght 1");
                    }
                }
            }

        #endregion

            #region constructs
            public SmppSubmitSm()
                : base()
            { }
            public SmppSubmitSm(byte[] incomigBytes)
                : base(incomigBytes)
            { }
            #endregion

            protected override void InitPdu()
            {
                base.InitPdu();
                DestinationAddressTon = Pdu.TonType.International;
                DestinationAddressNpi = Pdu.NpiType.ISDN;
                DestinationAddress = null;
                CommandStatus = 0;
                CommandID = CommandIdType.submit_sm;

            }

            public override void ToMsbHexEncoding()
            {
                ArrayList pdu = GetPduHeader();
                pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(ServiceType)));
                pdu.Add((byte)SourceAddressTon);
                pdu.Add((byte)SourceAddressNpi);
                pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(SourceAddress)));
                pdu.Add((byte)DestinationAddressTon);
                pdu.Add((byte)DestinationAddressNpi);
                pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(DestinationAddress)));
                pdu.AddRange(GetBytesAfterDesination());

                PacketBytes = EncodePduForTrasmission(pdu);
            }

            protected override void DecodeSmscResponse()
            {
                byte[] remainder = BytesAfterHeader;
                ServiceType = SmppStringUtil.GetCStringFromBody(ref remainder);
                SourceAddressTon = (TonType)remainder[0];
                SourceAddressNpi = (NpiType)remainder[1];
                SourceAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);
                DestinationAddressTon = (TonType)remainder[0];
                DestinationAddressNpi = (NpiType)remainder[1];
                DestinationAddress = SmppStringUtil.GetCStringFromBody(ref remainder, 2);
                EsmClass = remainder[0];
                ProtocolId = (SmppVersionType)remainder[1];
                PriorityFlag = (PriorityType)remainder[2];
                ScheduleDeliveryTime = SmppStringUtil.GetCStringFromBody(ref remainder, 3);
                ValidityPeriod = SmppStringUtil.GetCStringFromBody(ref remainder);
                RegisteredDelivery = (RegisteredDeliveryType)remainder[0];
                ReplaceIfPresentFlag = (remainder[1] == 0) ? false : true;
                DataCoding = (DataCodingType)remainder[2];
                SmDefaultMessageId = remainder[3];
                this.smLength = remainder[4];
                ShortMessage = SmppStringUtil.GetCStringFromBody(ref remainder, 5, 5 + SmLength);

                TranslateTlvDataIntoTable(remainder);
            }
        }
    }
    










        









   
