using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using SMPP.Utility;

namespace SMPP.Packet.Response
{
    public class SmppDataSmResp:SmppSubmitSmResp
    {
        public enum DeliveryFailureType : byte
        {
            DestinationUnavailable = 0x00,
            DestinationAddressInvalid = 0x01,
            PermanentNetworkError = 0x02,
            TemporaryNetworkError = 0x03
        }

        public DeliveryFailureType DeliveryFailureReason
        {
            get
            {
                return (DeliveryFailureType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.delivery_faulire_reason)[0];
            }
            set
            {
                SetOptionalParamBytes((UInt16)Pdu.OptionalParamCodes.delivery_faulire_reason, BitConverter.GetBytes(UnsignedNumConverter.SwapByteOrdering((byte)value)));
            }
        }

        public string NetworkErroCode
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.network_error_code);
            }
            set
            {
                PduUtil.SetNetworkErrorCode(this, value);
            }
        }

        public string AdditionalStatusInfoText
        {
            get
            {
                return GetOptionalParamString((ushort)Pdu.OptionalParamCodes.additional_status_info_text);
            }
            set
            {
                const int MAX_STATUS_LEN = 264;
                if (value == null)
                {
                    SetOptionalParamString((ushort)Pdu.OptionalParamCodes.additional_status_info_text, string.Empty);
                }
                else if (value.Length <= MAX_STATUS_LEN)
                {
                    SetOptionalParamString((ushort)Pdu.OptionalParamCodes.additional_status_info_text, value);
                }
                else
                {
                    throw new ArgumentException("additional_status_info_text must have length <=" + MAX_STATUS_LEN);
                }
            }
        }

        public DpfResultType DpfResult
        {
            get
            {
                return (DpfResultType)GetOptionalParamBytes((ushort)Pdu.OptionalParamCodes.dpf_result)[0];
            }
            set
            {
                SetOptionalParamBytes((UInt16)Pdu.OptionalParamCodes.dpf_result, BitConverter.GetBytes(UnsignedNumConverter.SwapByteOrdering((byte)value)));
            }
        }

        public SmppDataSmResp()
            : base()
        { }
        public SmppDataSmResp(byte[] incomingBytes)
            : base(incomingBytes)
        { }

        protected override void InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = CommandIdType.data_sm_resp;
        }

        public override void ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(MessageId)));

            PacketBytes = EncodePduForTrasmission(pdu);
        }

    }
}
