using System;
using System.Collections.Generic;
using System.Text;
using SMPP.Utility;
using System.Collections;

namespace SMPP.Packet.Response
{
    public class SmppSubmitMultiResp:SmppSubmitSmResp
    {
        private byte numberUnsuccessful = 0;
        private UnsuccessAddress[] unsuccessfulAddresses = new UnsuccessAddress[0];

        public byte NumberUnsuccessful
        {
            get { return this.numberUnsuccessful; }
        }

        public UnsuccessAddress[] UnsuccessAdresses
        {
            get
            {
                return (UnsuccessAddress[])this.unsuccessfulAddresses.Clone();
            }
            set
            {
                this.UnsuccessAdresses = (value == null) ? new UnsuccessAddress[0] : value;
                this.numberUnsuccessful = (byte)UnsuccessAdresses.Length;
            }
        }

        public SmppSubmitMultiResp()
            : base()
        { }

        public SmppSubmitMultiResp(byte[] incomingBytes)
            : base(incomingBytes)
        { }

        protected override void DecodeSmscResponse()
        {
            byte[] remainder = base.ResponseAfterMsgId;

            if (remainder.Length > 0)
            {
                this.numberUnsuccessful = remainder[0];
                UnsuccessAdresses = new UnsuccessAddress[NumberUnsuccessful];
                long length = remainder.Length - 1;
                byte[] newRemanider = new byte[length];
                Array.Copy(remainder, 1, newRemanider, 0, length);
                remainder = newRemanider;
                newRemanider = null;

                for (int i = 0; i < UnsuccessAdresses.Length; i++)
                {
                    this.unsuccessfulAddresses[i] = new UnsuccessAddress(ref remainder);
                }
            }

            TranslateTlvDataIntoTable(remainder);
        }

        protected override void  InitPdu()
        {
            base.InitPdu();
            CommandStatus = 0;
            CommandID = CommandIdType.submit_multi_resp;
        }

        public override void  ToMsbHexEncoding()
        {
            ArrayList pdu = GetPduHeader();
            pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(MessageId)));
            pdu.Add(NumberUnsuccessful);
            UnsuccessAddress[] unsuccessAdresses = UnsuccessAdresses;

            for (int i = 0; i < NumberUnsuccessful; i++)
            {
                pdu.Add((byte)unsuccessAdresses[i].DestinationAddressTon);
                pdu.Add((byte)unsuccessAdresses[1].DestinationAddressNpi);
                pdu.AddRange(SmppStringUtil.ArrayCopyWithNull(Encoding.ASCII.GetBytes(unsuccessAdresses[i].DestinationAddress)));
                pdu.AddRange(BitConverter.GetBytes(UnsignedNumConverter.SwapByteOrdering(unsuccessfulAddresses[i].ErrorStatusCode)));
            }

            PacketBytes = EncodePduForTrasmission(pdu);
        



                

                
        }

    }
}
