using System;
using System.Net.Sockets;
using System.Collections;
using System.Threading;
using SMPP.Packet.Request;
using SMPP.Utility;
using SMPP.Packet.Response;
using SMPP.Packet;
using SMPP.EventObject;
using System.Timers;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace SMPP 
{
	
	public class SMPPCommunicator
    {
        #region fields
        private AsyncSocketClient asClient;
		private Int16 port;
		private SmppBind.BindingType bindType;
		private string host;
		private Pdu.NpiType npiType;
		private Pdu.TonType tonType; 
		private SmppBind.SmppVersionType version;
		private string addressRange;
		private string password;
		private string systemId;
		private string systemType;
		private System.Timers.Timer timer;
		private int enquireLinkInterval;
		private int sleepTimeAfterSocketFailure;
		private bool sentUnbindPacket = true;  //default to true since we start out unbound
		private string username;
        #endregion

        #region properties

        public string Username
		{
			set
			{
				this.username = value;
			}
		}

		public bool SentUnbindPacket
		{
			get
			{
                return this.sentUnbindPacket;
			}
		}

		public Int16 Port
		{
			get
			{
                return this.port;
			}
			set
			{
				this.port = value;
			}
		}
		
		public SmppBind.BindingType BindType
		{
			get
			{
                return this.bindType;
			}
			set
			{
				this.bindType = value;
			}
		
		}

		public string SystemType
		{
			get
			{
				return this.systemType;
			}
			set
			{
				this.systemType = value;
			}
		}
		
		public string SystemId
		{
			get
			{
				return this.systemId;
			}
			set
			{
				this.systemId = value;
			}
		}

		public string Password
		{
			get
			{
				return this.password;
			}
			set
			{
				this.password = value;
			}
		}

		public string Host
		{
			get
			{
				return this.host;
			}
			set
			{
				this.host = value;
			}
		}
		public Pdu.NpiType NpiType 
		{
			get
			{
				return this.npiType;
			}
			set 
			{
				this.npiType = value;
			}
		}

		public Pdu.TonType TonType 
		{
			get
			{
				return this.tonType;
			}
			set 
			{
				this.tonType = value;
			}
		}

		public SmppBind.SmppVersionType Version 
		{
			get
			{
				return this.version;
			}
			set 
			{
				this.version = value;
			}
		}

		public string AddressRange 
		{
			get
			{
				return this.addressRange;
			}
			set 
			{
				this.addressRange = value;
			}
		}

    	public int EnquireLinkInterval
		{
			get 
			{
				return this.enquireLinkInterval;
			}

			set
			{
				if(value >= 0)
					this.enquireLinkInterval = value;
			}
		}

		public int SleepTimeAfterSocketFailure
		{
			get 
			{
				return this.sleepTimeAfterSocketFailure;
			}

			set
			{
				if(value >= 0)
					this.sleepTimeAfterSocketFailure = value;
			}
		}
		#endregion properties
		
		#region events
		
		public event BindRespEventHandler OnBindResp;
		
		public event ErrorEventHandler OnError;
		
		public event UnbindRespEventHandler OnUnboundResp;
		
		public event ClosingEventHandler OnClose;
		
		public event AlertEventHandler OnAlert;
		
		public event SubmitSmRespEventHandler OnSubmitSmResp;
		
		public event EnquireLinkRespEventHandler OnEnquireLinkResp;
		
		public event SubmitSmEventHandler OnSubmitSm;
		
		public event QuerySmEventHandler OnQuerySm;
		
		public event GenericNackEventHandler OnGenericNack;
		
		public event EnquireLinkEventHandler OnEnquireLink;
		
		public event UnbindEventHandler OnUnbind;
		
		public event BindEventHandler OnBind;
		
		public event CancelSmEventHandler OnCancelSm;
		
		public event CancelSmRespEventHandler OnCancelSmResp;
		
		public event QuerySmRespEventHandler OnQuerySmResp;
		
		public event DataSmEventHandler OnDataSm;
		
		public event DataSmRespEventHandler OnDataSmResp;
		
		public event DeliverSmEventHandler OnDeliverSm;
		
		public event DeliverSmRespEventHandler OnDeliverSmResp;
		
		public event ReplaceSmEventHandler OnReplaceSm;
		
		public event ReplaceSmRespEventHandler OnReplaceSmResp;
		
		public event SubmitMultiEventHandler OnSubmitMulti;
		
		public event SubmitMultiRespEventHandler OnSubmitMultiResp;
		#endregion events
		
		#region delegates

		
		public delegate void BindRespEventHandler(object source, BindRespEventArgs e);
		
		public delegate void ErrorEventHandler(object source, CommonErrorEventArgs e);
		
		public delegate void UnbindRespEventHandler(object source, UnbindRespEventArgs e);
		
		public delegate void ClosingEventHandler(object source, EventArgs e);
		
		public delegate void AlertEventHandler(object source, AlertEventArgs e);
		
		public delegate void SubmitSmRespEventHandler(object source, SubmitSmRespEventArgs e);
		
		public delegate void EnquireLinkRespEventHandler(object source, EnquireLinkRespEventArgs e);
		
		public delegate void SubmitSmEventHandler(object source, SubmitSmEventArgs e);
		
		public delegate void QuerySmEventHandler(object source, QuerySmEventArgs e);
		
		public delegate void GenericNackEventHandler(object source, GenericNackEventArgs e);
		
		public delegate void EnquireLinkEventHandler(object source, EnquireLinkEventArgs e);
		
		public delegate void UnbindEventHandler(object source, UnbindEventArgs e);
		
		public delegate void BindEventHandler(object source, BindEventArgs e);
		
		public delegate void CancelSmEventHandler(object source, CancelSmEventArgs e);
		
		public delegate void CancelSmRespEventHandler(object source, CancelSmRespEventArgs e);
		
		public delegate void QuerySmRespEventHandler(object source, QuerySmRespEventArgs e);
		
		public delegate void DataSmEventHandler(object source, DataSmEventArgs e);
		
		public delegate void DataSmRespEventHandler(object source, DataSmRespEventArgs e);
		
		public delegate void DeliverSmEventHandler(object source, DeliverSmEventArgs e);
		
		public delegate void DeliverSmRespEventHandler(object source, DeliverSmRespEventArgs e);
		
		public delegate void ReplaceSmEventHandler(object source, ReplaceSmEventArgs e);
		
		public delegate void ReplaceSmRespEventHandler(object source, ReplaceSmRespEventArgs e);
		
		public delegate void SubmitMultiEventHandler(object source, SubmitMultiEventArgs e);
		
		public delegate void SubmitMultiRespEventHandler(object source, SubmitMultiRespEventArgs e);
	
		#endregion delegates

		#region constructors
		
		
		public SMPPCommunicator()
		{
			InitCommunicator();
		}

        public SMPPCommunicator(Int16 port, SmppBind.BindingType bindType, string host, Pdu.NpiType npiType, Pdu.TonType tonType,
                            SmppBind.SmppVersionType version, string addressRange, string password, string systemId, string systemType,
                            int enquireLinkInterval, int sleepTimeAfterSocketFailure)
        {
            this.Port = port;
            this.BindType = bindType;
            this.Host = host;
            this.NpiType = npiType;
            this.TonType = tonType;
            this.Version = version;
            this.AddressRange = addressRange;
            this.Password = password;
            this.SystemId = systemId;
            this.SystemType = systemType;
            this.EnquireLinkInterval = enquireLinkInterval;
            this.SleepTimeAfterSocketFailure = sleepTimeAfterSocketFailure;
        }
		#endregion constructors

		public void SendPdu(Pdu packet)
		{
			bool sendFailed = true;
			
			while(sendFailed)
			{
				try
				{
					packet.ToMsbHexEncoding();
					asClient.Send(packet.PacketBytes);
					sendFailed = false;
				}
				catch(Exception exc)
				{
					if(OnError != null)
					{
						OnError(this, new CommonErrorEventArgs(exc));
					}

					//try to stay alive
					if((exc.Message.ToLower().IndexOf("socket is closed")>= 0 || 
						exc.Message.ToLower().IndexOf("unable to write data to the transport connection")>= 0))
					{
						System.Threading.Thread.Sleep(SleepTimeAfterSocketFailure * 1000);
						Bind();
					}
					else	
					{
						//don't know what happened, but kick out
						sendFailed = false;
					}
				}
			}
		}

		public void Bind()
		{
			try
			{
				if(asClient != null)
					asClient.Disconnect();
			}
			catch
			{
				//drop it on the floor
			}

			//connect
			try 
			{
				asClient = new AsyncSocketClient(10240, null,
					new AsyncSocketClient.MessageHandler(ClientMessageHandler),
					new AsyncSocketClient.SocketClosingHandler(ClientCloseHandler),
					new AsyncSocketClient.ErrorHandler(ClientErrorHandler));

				asClient.Connect(Host, Port);

				SmppBind request = new SmppBind();
				request.SystemId = SystemId;
				request.Password = Password;
				request.SystemType = SystemType;
				request.InterfaceVersion = Version;
				request.AddressTon = TonType;
				request.AddressNpi = NpiType;
				request.AddressRange = AddressRange;
				request.BindType = BindType;

				SendPdu(request);
				this.sentUnbindPacket = false;

				if(this.enquireLinkInterval > 0)
				{
					if(timer == null)
					{
						timer = new System.Timers.Timer();
						timer.Elapsed += new ElapsedEventHandler(TimerElapsed);
					}

					if(timer != null)		//reset the old timer
					{
						timer.Stop();

						timer.Interval = EnquireLinkInterval * 1000;
						timer.Start();
					}
				}
			} 
			catch(Exception exc)
			{
				if(OnError != null)
				{
					OnError(this, new CommonErrorEventArgs(exc));
				}
			}
		}

		public void Unbind()
		{
			if(timer != null)
				timer.Stop();
			
			if(!this.sentUnbindPacket)
			{
				SmppUnbind request = new SmppUnbind();
				SendPdu(request);
				 this.sentUnbindPacket = true;
			}
		}

		#region internal methods		
		
		internal void ClientMessageHandler(AsyncSocketClient client)
		{
			try 
			{
				Queue responseQueue = new PduFactory().GetPduQueue(client.Buffer);
				ThreadPool.QueueUserWorkItem(new WaitCallback(ProcessPduQueue), responseQueue);
			} 
			catch(Exception exception)
			{
				if(OnError != null)
				{
					CommonErrorEventArgs e = new CommonErrorEventArgs(exception);
					OnError(this, e);
				}
			}
		}

		
		internal void ClientCloseHandler(AsyncSocketClient client)
		{
			//fire off a closing event
			if(OnClose != null)
			{
				System.EventArgs e = new System.EventArgs();
				OnClose(this, e);
			}
		}

		
		internal void ClientErrorHandler(AsyncSocketClient client,
			Exception exception)
		{
			//fire off an error handler
			if(OnError != null)
			{
				CommonErrorEventArgs e = new CommonErrorEventArgs(exception);
				OnError(this, e);
			}
		}
		#endregion internal methods

		#region private methods
		
		private void ProcessPduQueue(object queueStateObj)
		{
			Queue responseQueue = queueStateObj as Queue;

			foreach(Pdu response in responseQueue)
			{
				//based on each Pdu, fire off an event
				if(response != null)
					FireEvents(response);
			}
		}
		
		private void TimerElapsed(object sender, ElapsedEventArgs ea)
		{
			SendPdu(new SmppEnquireLink());
		}


		private void FireEvents(Pdu response)
		{
			//here we go...
			if(response is SmppBindResp)
			{
				if(OnBindResp != null)
				{
					OnBindResp(this, new BindRespEventArgs((SmppBindResp)response));
				}
			} 
			else if(response is SmppUnbindResp)
			{
				//disconnect
				asClient.Disconnect();
				if(OnUnboundResp != null)
				{
					OnUnboundResp(this, new UnbindRespEventArgs((SmppUnbindResp)response));
				}
			} 
			else if(response is SmppAlertNotification)
			{
				if(OnAlert != null)
				{
					OnAlert(this, new AlertEventArgs((SmppAlertNotification)response));
				}
			}	
			else if(response is SmppSubmitSmResp)
			{
				if(OnSubmitSmResp != null)
				{
					OnSubmitSmResp(this,
						new SubmitSmRespEventArgs((SmppSubmitSmResp)response));
				}
			}
			else if(response is SmppEnquireLinkResp)
			{
				if(OnEnquireLinkResp != null)
				{
					OnEnquireLinkResp(this, new EnquireLinkRespEventArgs((SmppEnquireLinkResp)response));
				}
			}
			else if(response is SmppSubmitSm)
			{
				if(OnSubmitSm != null)
				{
					OnSubmitSm(this, new SubmitSmEventArgs((SmppSubmitSm)response));
				}
				else
				{
					//default a response
					SmppSubmitSmResp pdu = new SmppSubmitSmResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.MessageId = System.Guid.NewGuid().ToString().Substring(0, 10);
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppQuerySm)
			{
				if(OnQuerySm != null)
				{
					OnQuerySm(this, new QuerySmEventArgs((SmppQuerySm)response));
				}
				else
				{
					//default a response
					SmppQuerySmResp pdu = new SmppQuerySmResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppGenericNack)
			{
				if(OnGenericNack != null)
				{
					OnGenericNack(this, new GenericNackEventArgs((SmppGenericNack)response));
				}
			}
			else if(response is SmppEnquireLink)
			{
				if(OnEnquireLink != null)
				{
					OnEnquireLink(this, new EnquireLinkEventArgs((SmppEnquireLink)response));
				}
				
				//send a response back
				SmppEnquireLinkResp pdu = new SmppEnquireLinkResp();
				pdu.SequenceNumber = response.SequenceNumber;
				pdu.CommandStatus = 0;

				SendPdu(pdu);
			}
			else if(response is SmppUnbind)
			{
				if(OnUnbind != null)
				{
					OnUnbind(this, new UnbindEventArgs((SmppUnbind)response));
				}
				else
				{
					//default a response
					SmppUnbindResp pdu = new SmppUnbindResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppBind)
			{
				if(OnBind != null)
				{
					OnBind(this, new BindEventArgs((SmppBind)response));
				}
				else
				{
					//default a response
					SmppBindResp pdu = new SmppBindResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
					pdu.SystemId = "Generic";
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppCancelSm)
			{
				if(OnCancelSm != null)
				{
					OnCancelSm(this, new CancelSmEventArgs((SmppCancelSm)response));
				}
				else
				{
					//default a response
					SmppCancelSmResp pdu = new SmppCancelSmResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppCancelSmResp)
			{
				if(OnCancelSmResp != null)
				{
					OnCancelSmResp(this, new CancelSmRespEventArgs((SmppCancelSmResp)response));
				}
			}
			else if(response is SmppCancelSmResp)
			{
				if(OnCancelSmResp != null)
				{
					OnCancelSmResp(this, new CancelSmRespEventArgs((SmppCancelSmResp)response));
				}
			}
			else if(response is SmppQuerySmResp)
			{
				if(OnQuerySmResp != null)
				{
					OnQuerySmResp(this, new QuerySmRespEventArgs((SmppQuerySmResp)response));
				}
			}
			else if(response is SmppDataSm)
			{
				if(OnDataSm != null)
				{
					OnDataSm(this, new DataSmEventArgs((SmppDataSm)response));
				}
				else
				{
					//default a response
					SmppDataSmResp pdu = new SmppDataSmResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
					pdu.MessageId = "Generic";
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppDataSmResp)
			{
				if(OnDataSmResp != null)
				{
					OnDataSmResp(this, new DataSmRespEventArgs((SmppDataSmResp)response));
				}
			}
			else if(response is SmppDeliverSm)
			{
				if(OnDeliverSm != null)
				{
					OnDeliverSm(this, new DeliverSmEventArgs((SmppDeliverSm)response));
				}
				else
				{
					//default a response
					SmppDeliverSmResp pdu = new SmppDeliverSmResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppDeliverSmResp)
			{
				if(OnDeliverSmResp != null)
				{
					OnDeliverSmResp(this, new DeliverSmRespEventArgs((SmppDeliverSmResp)response));
				}
			}
			else if(response is SmppReplaceSm)
			{
				if(OnReplaceSm != null)
				{
					OnReplaceSm(this, new ReplaceSmEventArgs((SmppReplaceSm)response));
				}
				else
				{
					//default a response
					SmppReplaceSmResp pdu = new SmppReplaceSmResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppReplaceSmResp)
			{
				if(OnReplaceSmResp != null)
				{
					OnReplaceSmResp(this, new ReplaceSmRespEventArgs((SmppReplaceSmResp)response));
				}
			}
			else if(response is SmppSubmitMulti)
			{
				if(OnSubmitMulti != null)
				{
					OnSubmitMulti(this, new SubmitMultiEventArgs((SmppSubmitMulti)response));
				}
				else
				{
					//default a response
					SmppSubmitMultiResp pdu = new SmppSubmitMultiResp();
					pdu.SequenceNumber = response.SequenceNumber;
					pdu.CommandStatus = 0;
	
					SendPdu(pdu);
				}
			}
			else if(response is SmppSubmitMultiResp)
			{
				if(OnSubmitMultiResp != null)
				{
					OnSubmitMultiResp(this, new SubmitMultiRespEventArgs((SmppSubmitMultiResp)response));
				}
			}
		}
	
		
		private void InitCommunicator()
		{		
			Port = 3701;
			BindType = SmppBind.BindingType.BindAsTransceiver;
			Host = "10.96.211.50";
			NpiType = Pdu.NpiType.ISDN;
			TonType = Pdu.TonType.Unknown; 
			Version = SmppBind.SmppVersionType.Version3_4;
			AddressRange = null;
			Password = "smsc";
			SystemId = "SMPP1";
			SystemType = null;
			EnquireLinkInterval = 15;
			SleepTimeAfterSocketFailure = 10;
		}
		#endregion private methods
		

	}
}
