using System;
using SMPP.Packet;

namespace SMPP.Utility
{
	
	public sealed class DestinationAddress
	{
		private Pdu.TonType destinationAddressTon = Pdu.TonType.International;
		private Pdu.NpiType destinationAddressNpi = Pdu.NpiType.ISDN;
		private string destinationAddress = string.Empty;
		private string distributionList = string.Empty;
		private bool isDistributionList = false;
		
		#region properties
		
		public Pdu.TonType DestinationAddressTon
		{
			get
			{
				return destinationAddressTon;
			}
		}
		
		public Pdu.NpiType DestinationAddressNpi
		{
			get
			{
				return destinationAddressNpi;
			}
		}
		
		public string DestAddress
		{
			get
			{
				return destinationAddress;
			}
		}
		
		public string DistributionList
		{
			get
			{
				return distributionList;
			}
		}
		
		public bool IsDistributionList
		{
			get
			{
				return isDistributionList;
			}
		}
		
		#endregion properties
		
	
		public DestinationAddress(ref byte[] address)
		{
			if(address[0] == 0x01)
			{
				isDistributionList = false;
			}
			else if(address[0] == 0x02)
			{
				isDistributionList = true;
			}
			else
			{
				throw new ApplicationException("Unable to determine type of destination address");
			}
			
			if(!IsDistributionList)
			{
				destinationAddressTon = (Pdu.TonType)address[1];
				destinationAddressNpi = (Pdu.NpiType)address[2];
				destinationAddress = SmppStringUtil.GetCStringFromBody(ref address, 3);
//				
//				long length = address.Length - 4;
//				byte[] newRemainder = new byte[length];
//				Array.Copy(address, 4, newRemainder, 0, length);
//				//and change the reference
//				address = newRemainder;
//				newRemainder = null;
			}
			else
			{
				distributionList = SmppStringUtil.GetCStringFromBody(ref address, 1);
//				
//				long length = address.Length - 4;
//				byte[] newRemainder = new byte[length];
//				Array.Copy(address, 4, newRemainder, 0, length);
//				//and change the reference
//				address = newRemainder;
//				newRemainder = null;
			}
		}
		
		public DestinationAddress(
			Pdu.TonType destinationAddressTon, 
			Pdu.NpiType destinationAddressNpi, 
			string destinationAdress)
		{
			if(destinationAdress == null || destinationAdress.Length > 20)
			{
				throw new ArgumentException(
					"Destination Adress must be 20 characters or less.");
			}
			
			isDistributionList = false;
			
			this.destinationAddressTon = destinationAddressTon;
			this.destinationAddressNpi = destinationAddressNpi;
			this.destinationAddress = destinationAdress;
		}
		
	
		public DestinationAddress(string distributionList)
		{
			if(distributionList == null || distributionList.Length > 20)
			{
				throw new ArgumentException(
					"distribution list must be 20 characters or less.");
			}
			
			isDistributionList = true;
			
			this.distributionList = distributionList;
		}
		
	
		public object Clone()
		{
			DestinationAddress temp = null;
			
			if(!this.IsDistributionList)
			{
				temp = new DestinationAddress(
					destinationAddressTon, destinationAddressNpi, destinationAddress);
			}
			else
			{
				temp = new DestinationAddress(distributionList);
			}
			
			temp.isDistributionList = this.IsDistributionList;
			
			return temp;
		}
		
	
		public override bool Equals(object obj)
		{
	    if (obj == null)
	    {
	    	return false;
	    }
	
	    if (this.GetType() != obj.GetType())
	    {
	    	return false;
	    }
	
	    DestinationAddress da = (DestinationAddress) obj;
	
	    if(!da.IsDistributionList)
	    {
	    	// value member check
		    return 
		    	destinationAddressTon.Equals(da.destinationAddressTon) &&
		      destinationAddressNpi.Equals(da.destinationAddressNpi) &&
		      destinationAddress.Equals(da.destinationAddress);
	    }
	    else
	    {
	    	return distributionList.Equals(da.distributionList);
	    }
		}
		
	
	public override int GetHashCode()
    {
			int hashCode = 0;
			
			if(!this.IsDistributionList)
			{
				hashCode ^= this.DestAddress.GetHashCode();
				hashCode ^= this.DestinationAddressNpi.GetHashCode();
				hashCode ^= this.DestinationAddressTon.GetHashCode();
			}
			else
			{
				hashCode ^= this.DistributionList.GetHashCode();
			}

			return hashCode;
    }
	}
}
