using System;
using System.Collections;
using System.Text;
using SMPP.Packet;

namespace SMPP.Utility
{
	/// <summary>
	/// Holds common functionality for requests.
	/// </summary>
	public class PduUtil
	{			
		#region constants
		
		private const int SUBADDRESS_MIN = 2;
		private const int SUBADDRESS_MAX = 23;
		
		#endregion constants
		
		private PduUtil()
		{
		}
		
		
		public static byte InsertShortMessage(ArrayList pdu, object ShortMessage)
		{
			byte[] msg;
			
			if(ShortMessage == null)
			{
				msg = new Byte[]{0x00};
			}
			else if(ShortMessage is string)
			{
				msg =  Encoding.ASCII.GetBytes((string)ShortMessage);
			}
			else if(ShortMessage is byte[])
			{
				msg =(byte[])ShortMessage;
			}
			else
			{
				throw new ArgumentException("Short Message must be a string or byte array.");
			}
			//			if(msg.Length >= MessageLcd2.SHORT_MESSAGE_LIMIT)
			//				throw new ArgumentException(
			//					"Short message cannot be longer than " +
			//					MessageLcd2.SHORT_MESSAGE_LIMIT + " octets.");
			
			byte SmLength = (byte)msg.Length;
			pdu.Add(SmLength);
			pdu.AddRange(msg);
			
			return SmLength;
		}
		
		
		public static void SetReceiptedMessageId(Pdu pdu, string val)
		{
			const int MAX_RECEIPTED_ID_LEN = 65;
			if(val == null)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.receipted_message_id, string.Empty);
			}
			else if(val.Length <= MAX_RECEIPTED_ID_LEN)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.receipted_message_id, val);
			}
			else
			{
				throw new ArgumentException(
					"receipted_message_id must have length 1-" + MAX_RECEIPTED_ID_LEN);
			}
		}
		
		
		public static void SetNetworkErrorCode(Pdu pdu, string val)
		{
			const int ERR_CODE_LEN = 3;
			if(val == null || val.Length != ERR_CODE_LEN)
			{
				throw new ArgumentException("network_error_code must have length " + ERR_CODE_LEN);
			}
			else
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.network_error_code,val);
			}
		}
		
		
		public static void SetItsSessionInfo(Pdu pdu, string val)
		{
			const int MAX_ITS = 16;
			
			if(val == null)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.its_session_info, string.Empty);
			}
			else if(val.Length == MAX_ITS)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.its_session_info, val);
			}
			else
			{
				throw new ArgumentException("its_session_info must have length " + MAX_ITS);
			}
		}
		
		
		public static void SetDestSubaddress(Pdu pdu, string val)
		{
			if(val.Length >= SUBADDRESS_MIN && val.Length <= SUBADDRESS_MAX)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.dest_subaddress, val);
			}
			else
			{
				throw new ArgumentException(
					"Destination subaddress must be between " + SUBADDRESS_MIN + 
					" and " + SUBADDRESS_MAX + " characters.");
			}
		}
		
	
		public static void SetSourceSubaddress(Pdu pdu, string val)
		{
			if(val.Length >= SUBADDRESS_MIN && val.Length <= SUBADDRESS_MAX)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.source_subaddress, val);
			}
			else
			{
				throw new ArgumentException(
					"Source subaddress must be between " + SUBADDRESS_MIN + 
					" and " + SUBADDRESS_MAX + " characters.");
			}
		}
		
		
		public static void SetCallbackNum(Pdu pdu, string val)
		{
			const int CALLBACK_NUM_MIN = 4;
			const int CALLBACK_NUM_MAX = 19;
			if(val.Length >= CALLBACK_NUM_MIN && val.Length <= CALLBACK_NUM_MAX)
			{
				pdu.SetOptionalParamString(
					(ushort)Pdu.OptionalParamCodes.callback_num, val);
			}
			else
			{
				throw new ArgumentException(
					"callback_num size must be between " + CALLBACK_NUM_MIN + 
					" and " + CALLBACK_NUM_MAX + " characters.");
			}
		}
		
		
		public static void SetMessagePayload(Pdu pdu, object val)
		{
			if(val != null)
				{
					byte[] encodedValue;
					if(val is string)
					{
						encodedValue = Encoding.ASCII.GetBytes((string)val);
					}
					else if(val is byte[])
					{
						encodedValue =(byte[])val;
					}
					else
					{
						throw new ArgumentException("Message Payload must be a string or byte array.");
					}
					
					const int MAX_PAYLOAD_LENGTH = 64000;
					if(encodedValue.Length < MAX_PAYLOAD_LENGTH)
					{
						pdu.SetOptionalParamBytes(
							(ushort)Pdu.OptionalParamCodes.message_payload, encodedValue);
					}
					else
					{
						throw new ArgumentException(
							"Message Payload must be " + MAX_PAYLOAD_LENGTH + " characters or less in size.");
					}
				}
				else
				{
					pdu.SetOptionalParamBytes(
						(ushort)Pdu.OptionalParamCodes.message_payload, new byte[]{0});
				}
		}
	}
}
