using System;
using System.Text;

namespace SMPP.Utility
{
	/// <summary>
	/// Catchall utility class for dealing with strings in SMPP packets.
	/// </summary>
	public class SmppStringUtil
	{
		/// <summary>
		/// Do not instantiate.
		/// </summary>
		private SmppStringUtil()
		{
		}
		
		
		public static string GetStringFromBody(ref byte[] remainder, int startIndex, int endIndex)
		{
			string result;
			
			//make sure we aren't trying to read non-command
			//data
			if(endIndex <= remainder.Length)
			{
				result = System.Text.Encoding.ASCII.GetString(remainder, startIndex, endIndex - startIndex);
			}
			else
			{
				result = string.Empty;
			}
			
			//now trim down the remainder-no null character
			long length = remainder.Length - endIndex;
			if(length >= 0)
			{
				byte[] newRemainder = new byte[length];
				Array.Copy(remainder, endIndex, newRemainder, 0, length);
				remainder = newRemainder;
				newRemainder = null;
			}
			
			return result;
		}
		
		
		public static string GetCStringFromBody(ref byte[] remainder)
		{
			return GetCStringFromBody(ref remainder, 0);
		}
		
		
		public static string GetCStringFromBody(ref byte[] remainder, int startIndex)
		{
			int i;
			
			//find where the null character(end of string)is
			//this stops ON the null character
			for(i = startIndex; i < remainder.Length && remainder[i] != 0x00; i++)
			{
				;
			}
			
			return GetCStringFromBody(ref remainder, startIndex, i);
		}
		
		
		public static string GetCStringFromBody(ref byte[] remainder, int startIndex, int endIndex)
		{
			string result;
			
			//make sure we aren't trying to read non-command
			//data
			if(endIndex <= remainder.Length)
			{
				result = Encoding.ASCII.GetString(remainder, startIndex, endIndex - startIndex);
			}
			else
			{
				result = string.Empty;
			}
			
			//now trim down the remainder-chop off the null character as well
			long length = remainder.Length - endIndex - 1;
			if(length >= 0)
			{
				byte[] newRemainder = new byte[length];
				Array.Copy(remainder, endIndex + 1, newRemainder, 0, length);
				remainder = newRemainder;
				newRemainder = null;
			}
			
			return result;
		}
		
		
		public static byte[] ArrayCopyWithNull(byte[] source)
		{
			byte[] temp = new byte[source.Length + 1];
			for(int i = 0; i < source.Length; i++)
			{
				temp[i] = source[i];
			}
			//append the null
			temp[temp.Length - 1] =(byte)0x00;
			
			return temp;
		}
	}
}
