using System;
using SMPP.Packet;

namespace SMPP.Utility
{
	/// <summary>
	/// Defines an Unsuccess address, used with the response to submit_multi.
	/// The spec states that both SME addresses and distribution lists can be
	/// used here, but it only defines for the SME address, so that is all that
	/// this will handle.
	/// </summary>
	public class UnsuccessAddress
	{
		private Pdu.TonType destinationAddressTon;
		private Pdu.NpiType destinationAddressNpi;
		private string destinationAddress;
		private uint errorStatusCode;
		
		#region properties
		
		
		public Pdu.TonType DestinationAddressTon
		{
			get
			{
				return destinationAddressTon;
			}
		}
		
		
		public Pdu.NpiType DestinationAddressNpi
		{
			get
			{
				return destinationAddressNpi;
			}
		}
		
		
		public string DestinationAddress
		{
			get
			{
				return destinationAddress;
			}
		}
		
		
		public UInt32 ErrorStatusCode
		{
			get
			{
				return errorStatusCode;
			}
		}
		
		#endregion properties
		
		
		public UnsuccessAddress(ref byte[] address)
		{
			destinationAddressTon = (Pdu.TonType)address[0];
			destinationAddressNpi = (Pdu.NpiType)address[1];
			destinationAddress = SmppStringUtil.GetCStringFromBody(ref address, 2);
			//convert error status to host order
			errorStatusCode = UnsignedNumConverter.SwapByteOrdering(
									 BitConverter.ToUInt32(address, 0));
			//now we have to trim off four octets to account for the status code
			long length = address.Length - 4;
			byte[] newRemainder = new byte[length];
			Array.Copy(address, 4, newRemainder, 0, length);
			//and change the reference
			address = newRemainder;
			newRemainder = null;
		}
		
		
		public UnsuccessAddress(
			Pdu.TonType destinationAddressTon, 
			Pdu.NpiType destinationAddressNpi, 
			string destinationAdress, 
			UInt32 ErrorStatusCode)
		{
			this.destinationAddressTon = destinationAddressTon;
			this.destinationAddressNpi = destinationAddressNpi;
			this.destinationAddress = destinationAdress;
			this.errorStatusCode = ErrorStatusCode;
		}
		
		
		public object Clone()
		{
			UnsuccessAddress temp = new UnsuccessAddress(
				destinationAddressTon, destinationAddressNpi, destinationAddress, errorStatusCode);
			return temp;
		}
		
		
		public override bool Equals(object obj)
		{
	    if (obj == null)
	    {
	    	return false;
	    }
	
	    if (this.GetType() != obj.GetType())
	    {
	    	return false;
	    }
	
	    // safe because of the GetType check
	    UnsuccessAddress us = (UnsuccessAddress) obj;
	
	    // value member check
	    return 
	    	destinationAddressTon.Equals(us.destinationAddressTon) &&
	      destinationAddressNpi.Equals(us.destinationAddressNpi) &&
	      destinationAddress.Equals(us.destinationAddress) &&
	     	errorStatusCode.Equals(us.errorStatusCode);
		}
		
		
		public override int GetHashCode()
    {
			int hashCode = 0;
			hashCode ^= this.DestinationAddress.GetHashCode();
			hashCode ^= this.DestinationAddressNpi.GetHashCode();
			hashCode ^= this.DestinationAddressTon.GetHashCode();
			hashCode ^= this.ErrorStatusCode.GetHashCode();

			return hashCode;
    }
	}
}
