using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using SMPP;
using SMPP.Packet;
using SMPP.Packet.Request;
using SMPP.Packet.Response;
using System.Text.RegularExpressions;
using System.Threading;
using System.IO;



namespace SMSPlatform
{
    public partial class SMSPlatform : Form
    {
        private SqlConnection conn;
        private DataSet dataSetWeather;
        private SMPPCommunicator smpp;
        private Configuration config;
        private string configPath;
        private StreamWriter writer;
        

        public SMSPlatform()
        {
            InitializeComponent();

            smpp = new SMPPCommunicator();
            config = new Configuration();
            string applicationPath = Application.ExecutablePath;
            configPath = applicationPath.Replace("exe", "config");
            config = Configuration.Deserialize(@configPath);
            
            
            smpp.OnBindResp +=new SMPPCommunicator.BindRespEventHandler(smpp_OnBindResp);
            smpp.OnUnboundResp += new SMPPCommunicator.UnbindRespEventHandler(smpp_UnboundResp);
            smpp.OnSubmitSmResp +=new SMPPCommunicator.SubmitSmRespEventHandler(smpp_OnSubmitSmResp);
            smpp.OnDeliverSm +=new SMPPCommunicator.DeliverSmEventHandler(smpp_OnDeliverSm);
       
           
        }

        private void SMSPlatform_Load(object sender, EventArgs e)
        {
            dataSetWeather = new DataSet();

            try
            {

                conn = new SqlConnection(config.ConnectionString);
                conn.Open();

                writer = new StreamWriter(config.LogPath, true);

            }
            catch (Exception ex)
            {
                writer.WriteLine("[{0:dd.MM.yyyy}" + "|" + "{1:HH:mm:ss}]" + ex.Message, DateTime.Now, DateTime.Now);
                conn.Close();
                writer.Close();
            }

            timerWeather.Enabled = true;
        }

        private void smpp_OnBindResp(object sender, SMPP.EventObject.BindRespEventArgs args)
        {
            uint status = args.ResponsePdu.CommandStatus;
            if (status == 0)
            {
                toolStripStatusLabel.Text = "Connected";
                writer.WriteLine("[{0:dd.MM.yyyy}" + "|" + "{1:HH:mm:ss}]" + "Start application", DateTime.Now, DateTime.Now);
                writer.Flush();
            }
            if (status == 14) toolStripStatusLabel.Text = "Invalid Password";
        }

        private void smpp_UnboundResp(object sender, SMPP.EventObject.UnbindRespEventArgs args)
        {
            uint status = args.ResponsePdu.CommandStatus;
            if (status == 0) toolStripStatusLabel.Text = "Disconnected";
        }
        private void smpp_OnSubmitSmResp(object sender, SMPP.EventObject.SubmitSmRespEventArgs args)
        {

        }
        private void smpp_OnDeliverSm(object sender, SMPP.EventObject.DeliverSmEventArgs args)
        {
            ThreadStart starter = delegate { SMSService(args); };
            new Thread(starter).Start();
        
        }
        private void Startbutton_Click(object sender, EventArgs e)
        {
            smpp.Host = config.SMPP_Host_Address;
            smpp.Port = config.SMPP_Port;
            smpp.SystemId = config.SMPP_System_ID;
            smpp.Password = config.SMPP_System_Password;
            smpp.EnquireLinkInterval = config.EnquireLinkInterval;
            smpp.BindType = SmppBind.BindingType.BindAsTransceiver;

            smpp.Bind();
            
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutDialog aboutDialog = new AboutDialog();
            if(aboutDialog.ShowDialog() == DialogResult.OK)
            {
                aboutDialog.Close();
            }

            aboutDialog.Dispose();
            
        }

        private void configurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ConfigurationDialog configuration = new ConfigurationDialog();
            
            configuration.SMSCAddressTextBox.Text = config.SMPP_Host_Address;
            configuration.SMSCPortTextBox.Text = config.SMPP_Port.ToString();
            configuration.SystemIDTextBox.Text = config.SMPP_System_ID;
            configuration.PasswordTextBox.Text = config.SMPP_System_Password;
            configuration.NumberTextBox.Text = config.Number.ToString();

            if (configuration.ShowDialog() == DialogResult.OK)
            {
                config.SMPP_Host_Address = configuration.SMSCAddressTextBox.Text;
                config.SMPP_Port = Int16.Parse(configuration.SMSCPortTextBox.Text);
                config.SMPP_System_ID = configuration.SystemIDTextBox.Text;
                config.SMPP_System_Password = configuration.PasswordTextBox.Text;
                config.Number = Int32.Parse(configuration.NumberTextBox.Text);
                Configuration.Serialize(@configPath, config);
 
                configuration.Close();
                
            }

            configuration.Dispose();

        }

        private void Stopbutton_Click(object sender, EventArgs e)
        {
            smpp.Unbind();
            writer.WriteLine("[{0:dd.MM.yyyy}" + "|" + "{1:HH:mm:ss}]" + "Stop application", DateTime.Now, DateTime.Now);
            writer.Flush();
        }

        private void timerWeather_Tick(object sender, EventArgs e)
        {
            Thread threadWeather = new Thread(new ThreadStart(UpdateWeather));
            threadWeather.Start();
        }

        #region Methods

        public static bool Match(string text, string pattern)
        {
            Regex regex = new Regex(pattern);
            Match match = regex.Match(text.Trim().ToLower());

            if (match.Success) return true;
            else return false;
        }

        public SmppSubmitSm SendSMS(Pdu.TonType sourceTon,Pdu.NpiType sourceNpi,string sourceAddress,Pdu.TonType destinationTon,Pdu.NpiType destinationNpi,string destinationAddress,string shortMessage,Pdu.DataCodingType dcs)
        {
            SmppSubmitSm submit = new SmppSubmitSm();

            submit.SourceAddressTon = sourceTon;
            submit.SourceAddressNpi = sourceNpi;
            submit.SourceAddress = sourceAddress;
            submit.DestinationAddressTon = destinationTon;
            submit.DestinationAddressNpi = destinationNpi;
            submit.DestinationAddress = destinationAddress;
            submit.ShortMessage = shortMessage;
            submit.DataCoding = dcs;

            smpp.SendPdu(submit);

            return submit;
        }

        public void InsertTableSMS(SqlConnection connection,Pdu.TonType sourceTon,Pdu.NpiType sourceNpi,long sourceAddress, Pdu.TonType destinationTon,
            Pdu.NpiType destinationNpi, long destinationAddress, Pdu.DataCodingType dcs, DateTime time, string shortMessage)
        {
            try
            {
                SqlCommand command = new SqlCommand("InsertTableSMS", connection);
                command.CommandType = CommandType.StoredProcedure;

                command.Parameters.Add("@sourceTon", SqlDbType.TinyInt).Value = sourceTon;
                command.Parameters.Add("@sourceNpi", SqlDbType.TinyInt).Value = sourceNpi;
                command.Parameters.Add("@sourceAddress", SqlDbType.BigInt).Value = sourceAddress;
                command.Parameters.Add("@destTon", SqlDbType.TinyInt).Value = destinationTon;
                command.Parameters.Add("@destNpi", SqlDbType.TinyInt).Value = destinationNpi;
                command.Parameters.Add("@destAddress", SqlDbType.BigInt).Value = destinationAddress;
                if (dcs == Pdu.DataCodingType.SMSCDefault)
                    command.Parameters.Add("@dcs", SqlDbType.TinyInt).Value = 0;
                if (dcs == Pdu.DataCodingType.UCS2)
                    command.Parameters.Add("@dcs", SqlDbType.TinyInt).Value = 8;
                command.Parameters.Add("@date", SqlDbType.DateTime).Value = time;
                command.Parameters.Add("@shortMessage", SqlDbType.VarChar).Value = shortMessage;

                command.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                writer.WriteLine("[{0:dd.MM.yyyy}" + "|" + "{1:HH:mm:ss}]" + ex.Message, DateTime.Now, DateTime.Now);
                writer.Flush();
            }

        }
        public void UpdateWeather()
        {
            dataSetWeather.Clear();
            SqlDataAdapter dbAdapterWeather = new SqlDataAdapter("SELECT * FROM TableWeather", conn);
            dbAdapterWeather.Fill(dataSetWeather);
            dbAdapterWeather.Dispose();
        }
        public void SMSService(SMPP.EventObject.DeliverSmEventArgs args)
        {
            DateTime deliveredTime = DateTime.Now;
            DateTime submitedTime = new DateTime();

            SmppDeliverSmResp deliverSmResp = new SmppDeliverSmResp();
            deliverSmResp.SequenceNumber = args.DeliverSmPdu.SequenceNumber;
            smpp.SendPdu(deliverSmResp);

            SmppSubmitSm submit = new SmppSubmitSm();

            string text = args.DeliverSmPdu.ShortMessage.ToString();

            bool flag = false;
            foreach (DataTable tableWeather in dataSetWeather.Tables)
                foreach (DataRow rowWeather in tableWeather.Rows)
                {
                    if (Match(text, rowWeather[1].ToString()) == true)
                    {
                        submit = SendSMS(Pdu.TonType.Unknown, Pdu.NpiType.ISDN, config.Number.ToString(), Pdu.TonType.International, Pdu.NpiType.ISDN,
                            args.DeliverSmPdu.SourceAddress, rowWeather[2].ToString(), Pdu.DataCodingType.SMSCDefault);
                        submitedTime = DateTime.Now;
                        
                        flag = true;
                    }
                }

            if (flag == false)
            {
                submit = SendSMS(Pdu.TonType.Unknown, Pdu.NpiType.ISDN, config.Number.ToString(), Pdu.TonType.International, Pdu.NpiType.ISDN,
                           args.DeliverSmPdu.SourceAddress, config.FailSMS, Pdu.DataCodingType.SMSCDefault);
                submitedTime = DateTime.Now;
               
            }

            try
            {
                SqlConnection connection = new SqlConnection(config.ConnectionString);
                connection.Open();

                InsertTableSMS(connection, args.DeliverSmPdu.SourceAddressTon, args.DeliverSmPdu.SourceAddressNpi, long.Parse(args.DeliverSmPdu.SourceAddress.ToString()),
                    args.DeliverSmPdu.DestinationAddressTon, args.DeliverSmPdu.DestinationAddressNpi, long.Parse(args.DeliverSmPdu.DestinationAddress.ToString()),
                    args.DeliverSmPdu.DataCoding, deliveredTime, args.DeliverSmPdu.ShortMessage);

                InsertTableSMS(connection, submit.SourceAddressTon, submit.SourceAddressNpi, long.Parse(submit.SourceAddress.ToString()),
                    submit.DestinationAddressTon, submit.DestinationAddressNpi, long.Parse(submit.DestinationAddress.ToString()),
                    submit.DataCoding, submitedTime, submit.ShortMessage.ToString());


                connection.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        #endregion

    }
}